﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclCommon;
using wclWiFi;

namespace NetworkListManager
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclNetworkListManager wclNetworkListManager;

        internal class NetworkItem
        {
            public String Id { get; set; }
            public String Category { get; set; }
            public String Connectivity { get; set; }
            public String Description { get; set; }
            public String DomainType { get; set; }
            public String Name { get; set; }
            public String CreationDateTime { get; set; }
            public String ConnectedDateTime { get; set; }
            public String Connected { get; set; }
            public String ConnectedToInternet { get; set; }
        };

        internal class ConnectionItem
        {
            public String AdapterId { get; set; }
            public String Connected { get; set; }
            public String ConnectedToInternet { get; set; }
            public String Connectivity { get; set; }
            public String DomainType { get; set; }
            public String Id { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            cbNetworkType.SelectedIndex = 2;
            cbCategory.SelectedIndex = 0;

            wclNetworkListManager = new wclNetworkListManager();
            wclNetworkListManager.AfterOpen += new EventHandler(wclNetworkListManager_AfterOpen);
            wclNetworkListManager.BeforeClose +=new EventHandler(wclNetworkListManager_BeforeClose);
            wclNetworkListManager.OnConnectivityChanged += new wclNlmConnectivityChangeEvent(wclNetworkListManager_OnConnectivityChanged);
            wclNetworkListManager.OnNetworkAdded += new wclNlmNetworkEvent(wclNetworkListManager_OnNetworkAdded);
            wclNetworkListManager.OnNetworkConnectivityChanged += new wclNlmNetworkConnectivityChangedEvent(wclNetworkListManager_OnNetworkConnectivityChanged);
            wclNetworkListManager.OnNetworkDeleted += new wclNlmNetworkEvent(wclNetworkListManager_OnNetworkDeleted);
            wclNetworkListManager.OnNetworkPropertyChanged += new wclNlmNetworkPropertyChangedEvent(wclNetworkListManager_OnNetworkPropertyChanged);
            wclNetworkListManager.OnConnectionConnectivityChanged += new wclNlmConnectionConnectivityChangedEvent(wclNetworkListManager_OnConnectionConnectivityChanged);
            wclNetworkListManager.OnConnectionPropertyChanged += new wclNlmConnectionPropertyChangedEvent(wclNetworkListManager_OnConnectionPropertyChanged);
        }

        void wclNetworkListManager_AfterOpen(object sender, EventArgs e)
        {
            lbEvents.Items.Add("AfterOpen");
        }

        void wclNetworkListManager_BeforeClose(object sender, EventArgs e)
        {
            lbEvents.Items.Add("BeforeClose");
        }

        void wclNetworkListManager_OnConnectivityChanged(object Sender, wclNlmConnectivity Connectivity)
        {
            lbEvents.Items.Add("Connectivity changed: " + Connectivity.ToString());
        }

        void wclNetworkListManager_OnNetworkAdded(object Sender, Guid NetworkId)
        {
            lbEvents.Items.Add("Network added: " + NetworkId.ToString());
        }

        void wclNetworkListManager_OnNetworkConnectivityChanged(object Sender, Guid NetworkId, wclNlmConnectivity Connectivity)
        {
            lbEvents.Items.Add("Network " + NetworkId.ToString() + " connectivity changed: " + Connectivity.ToString());
        }

        void wclNetworkListManager_OnNetworkDeleted(object Sender, Guid NetworkId)
        {
            lbEvents.Items.Add("Network deleted: " + NetworkId.ToString());
        }

        void wclNetworkListManager_OnNetworkPropertyChanged(object Sender, Guid NetworkId, wclNlmNetworkPropertyChange Change)
        {
            lbEvents.Items.Add("Network " + NetworkId.ToString() + " property changed: " + Change.ToString());
        }

        void wclNetworkListManager_OnConnectionConnectivityChanged(object Sender, Guid ConnectionId, wclNlmConnectivity Connectivity)
        {
            lbEvents.Items.Add("Connection: " + ConnectionId.ToString() + " connectivity changed: " + Connectivity.ToString());
        }

        void wclNetworkListManager_OnConnectionPropertyChanged(object Sender, Guid ConnectionId, wclNlmConnectionProperty Prop)
        {
            lbEvents.Items.Add("Connection: " + ConnectionId.ToString() + " property changed: " + Prop.ToString());
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            wclNetworkListManager.Close();
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void btOpen_Click(object sender, RoutedEventArgs e)
        {
            ShowError(wclNetworkListManager.Open());
        }

        private void btClose_Click(object sender, RoutedEventArgs e)
        {
            ShowError(wclNetworkListManager.Close());
        }

        private Boolean ShowError(Int32 Error)
        {
            if (Error != wclErrors.WCL_E_SUCCESS)
            {
                MessageBox.Show("Error: 0x" + Error.ToString("X8"));
                return true;
            }

            return false;
        }

        private void btIsConnected_Click(object sender, RoutedEventArgs e)
        {
            bool Connected;
            if (!ShowError(wclNetworkListManager.GetConnected(out Connected)))
            {
                if (Connected)
                    MessageBox.Show("Connected");
                else
                    MessageBox.Show("NOT Connected");
            }
        }

        private void btIsInternet_Click(object sender, RoutedEventArgs e)
        {
            bool Connected;
            if (!ShowError(wclNetworkListManager.GetConnectedToInternet(out Connected)))
            {
                if (Connected)
                    MessageBox.Show("Connected");
                else
                    MessageBox.Show("NOT Connected");
            }
        }

        private void btConnectivity_Click(object sender, RoutedEventArgs e)
        {
            wclNlmConnectivity Connectivity;
            if (!ShowError(wclNetworkListManager.GetConnectivity(out Connectivity)))
                MessageBox.Show("Connectivity: " + Connectivity.ToString());
        }

        private void btEnumNetworks_Click(object sender, RoutedEventArgs e)
        {
            lvNetworks.Items.Clear();

            wclNlmEnumNetwork Flag = (wclNlmEnumNetwork)cbNetworkType.SelectedIndex;
            wclNlmNetwork[] Networks;
            Int32 Res = wclNetworkListManager.GetNetworks(Flag, out Networks);
            try
            {
                if (!ShowError(Res))
                {
                    for (int i = 0; i < Networks.Length; i++)
                    {
                        wclNlmNetwork Network = Networks[i];
                        NetworkItem Item = new NetworkItem();

                        Guid Id;
                        Res = Network.GetNetworkId(out Id);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.Id = Id.ToString();
                        else
                            Item.Id = "Error: 0x" + Res.ToString("X8");

                        wclNlmNetworkCategory Category;
                        Res = Network.GetCategory(out Category);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.Category = Category.ToString();
                        else
                            Item.Category = "Error: 0x" + Res.ToString("X8");

                        wclNlmConnectivity Connectivity;
                        Res = Network.GetConnectivity(out Connectivity);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.Connectivity = Connectivity.ToString();
                        else
                            Item.Connectivity = "Error: 0x" + Res.ToString("X8");

                        String Description;
                        Res = Network.GetDescription(out Description);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.Description = Description;
                        else
                            Item.Description = "Error: 0x" + Res.ToString("X8");

                        wclNlmDomainType Domain;
                        Res = Network.GetDomainType(out Domain);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.DomainType = Domain.ToString();
                        else
                            Item.DomainType = "Error: 0x" + Res.ToString("X8");

                        String AName;
                        Res = Network.GetName(out AName);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.Name = AName;
                        else
                            Item.Name = "Error: 0x" + Res.ToString("X8");

                        DateTime CreationDateTime;
                        DateTime ConnectedDateTime;
                        Res = Network.GetTimeCreatedAndConnected(out CreationDateTime,
                            out ConnectedDateTime);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                        {
                            Item.CreationDateTime = CreationDateTime.ToString();
                            Item.ConnectedDateTime = ConnectedDateTime.ToString();
                        }
                        else
                        {
                            Item.CreationDateTime = "Error: 0x" + Res.ToString("X8");
                            Item.ConnectedDateTime = "Error: 0x" + Res.ToString("X8");
                        }

                        bool Connected;
                        Res = Network.GetConnected(out Connected);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.Connected = Connected.ToString();
                        else
                            Item.Connected = "Error: 0x" + Res.ToString("X8");

                        Res = Network.GetConnectedToInternet(out Connected);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.ConnectedToInternet = Connected.ToString();
                        else
                            Item.ConnectedToInternet = "Error: 0x" + Res.ToString("X8");

                        lvNetworks.Items.Add(Item);
                    }
                }
            }
            finally
            {
                Networks = null;
            }
        }

        private void btRefresh_Click(object sender, RoutedEventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            NetworkItem Item = (NetworkItem)lvNetworks.SelectedItems[0];
            Guid Id = new Guid(Item.Id);
            if (!ShowError(wclNetworkListManager.GetNetwork(Id, out Network)))
            {
                try
                {
                    wclNlmNetworkCategory Category;
                    int Res = Network.GetCategory(out Category);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.Category = Category.ToString();
                    else
                        Item.Category = "Error: 0x" + Res.ToString("X8");

                    wclNlmConnectivity Connectivity;
                    Res = Network.GetConnectivity(out Connectivity);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.Connectivity = Connectivity.ToString();
                    else
                        Item.Connectivity = "Error: 0x" + Res.ToString("X8");

                    String Description;
                    Res = Network.GetDescription(out Description);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.Description = Description;
                    else
                        Item.Description = "Error: 0x" + Res.ToString("X8");

                    wclNlmDomainType Domain;
                    Res = Network.GetDomainType(out Domain);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.DomainType = Domain.ToString();
                    else
                        Item.DomainType = "Error: 0x" + Res.ToString("X8");

                    String AName;
                    Res = Network.GetName(out AName);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.Name = AName;
                    else
                        Item.Name = "Error: 0x" + Res.ToString("X8");

                    DateTime CreationDateTime;
                    DateTime ConnectedDateTime;
                    Res = Network.GetTimeCreatedAndConnected(out CreationDateTime,
                        out ConnectedDateTime);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                    {
                        Item.CreationDateTime = CreationDateTime.ToString();
                        Item.ConnectedDateTime = ConnectedDateTime.ToString();
                    }
                    else
                    {
                        Item.CreationDateTime = "Error: 0x" + Res.ToString("X8");
                        Item.ConnectedDateTime = "Error: 0x" + Res.ToString("X8");
                    }

                    bool Connected;
                    Res = Network.GetConnected(out Connected);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.Connected = Connected.ToString();
                    else
                        Item.Connected = "Error: 0x" + Res.ToString("X8");

                    Res = Network.GetConnectedToInternet(out Connected);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.ConnectedToInternet = Connected.ToString();
                    else
                        Item.ConnectedToInternet = "Error: 0x" + Res.ToString("X8");

                    lvNetworks.Items.Refresh();
                }
                finally
                {
                    Network = null;
                }
            }
        }

        private void btSetName_Click(object sender, RoutedEventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            int Res = wclNetworkListManager.GetNetwork(new Guid(((NetworkItem)lvNetworks.SelectedItems[0]).Id),
                out Network);
            if (!ShowError(Res))
            {
                if (ShowError(Network.SetName(edNewNameOrDescription.Text)))
                    MessageBox.Show("Admin privilegies required");

                Network = null;
            }
        }

        private void btSetDescription_Click(object sender, RoutedEventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            int Res = wclNetworkListManager.GetNetwork(new Guid(((NetworkItem)lvNetworks.SelectedItems[0]).Id),
                out Network);
            if (!ShowError(Res))
            {
                if (ShowError(Network.SetDescription(edNewNameOrDescription.Text)))
                    MessageBox.Show("Admin privilegies required");

                Network = null;
            }
        }

        private void btSetCategory_Click(object sender, RoutedEventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            int Res = wclNetworkListManager.GetNetwork(new Guid(((NetworkItem)lvNetworks.SelectedItems[0]).Id),
                out Network);
            if (!ShowError(Res))
            {
                if (ShowError(Network.SetCategory((wclNlmNetworkCategory)cbCategory.SelectedIndex)))
                    MessageBox.Show("Admin privilegies required");

                Network = null;
            }
        }

        private void ShowConnections(ref wclNlmConnection[] Connections)
        {
            lvConnection.Items.Clear();

            if (Connections == null)
                return;

            try
            {
                for (int i = 0; i < Connections.Length; i++)
                {
                    wclNlmConnection Connection = Connections[i];
                    ConnectionItem Item = new ConnectionItem();

                    Guid Id;
                    int Res = Connection.GetAdapterId(out Id);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.AdapterId = Id.ToString();
                    else
                        Item.AdapterId = "Error: 0x" + Res.ToString("X8");

                    bool Connected;
                    Res = Connection.GetConnected(out Connected);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.Connected = Connected.ToString();
                    else
                        Item.Connected = "Error: 0x" + Res.ToString("X8");

                    Res = Connection.GetConnectedToInternet(out Connected);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.ConnectedToInternet = Connected.ToString();
                    else
                        Item.ConnectedToInternet = "Error: 0x" + Res.ToString("X8");

                    wclNlmConnectivity Connectivity;
                    Res = Connection.GetConnectivity(out Connectivity);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.Connectivity  = Connectivity.ToString();
                    else
                        Item.Connectivity = "Error: 0x" + Res.ToString("X8");

                    wclNlmDomainType Domain;
                    Res = Connection.GetDomainType(out Domain);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.DomainType  = Domain.ToString();
                    else
                        Item.DomainType = "Error: 0x" + Res.ToString("X8");

                    Res = Connection.GetId(out Id);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.Id  = Id.ToString();
                    else
                        Item.Id = "Error: 0x" + Res.ToString("X8");

                    lvConnection.Items.Add(Item);
                }
            }
            finally
            {
                Connections = null;
            }
        }

        private void btGetMachineConnections_Click(object sender, RoutedEventArgs e)
        {
            wclNlmConnection[] Connections;
            if (!ShowError(wclNetworkListManager.GetConnections(out Connections)))
                ShowConnections(ref Connections);
        }

        private void btGetNetworkConnections_Click(object sender, RoutedEventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            int Res = wclNetworkListManager.GetNetwork(new Guid(((NetworkItem)lvNetworks.SelectedItems[0]).Id),
                out Network);
            if (!ShowError(Res))
            {
                try
                {
                    wclNlmConnection[] Connections;
                    if (!ShowError(Network.GetConnections(out Connections)))
                        ShowConnections(ref Connections);
                }
                finally
                {
                    Network = null;
                }
            }
        }

        private void btDelete_Click(object sender, RoutedEventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            int Res = wclNetworkListManager.DeleteNetwork(new Guid(((NetworkItem)lvNetworks.SelectedItems[0]).Id));
            if (!ShowError(Res))
                lvNetworks.Items.Remove(lvNetworks.SelectedItems[0]);
        }
    }
}
