﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclCommon;
using wclWiFi;

namespace WiFiDirect
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclWiFiDirectAdvertiser wclWiFiDirectAdvertiser;

        internal class DeviceItem
        {
            public String DeviceName { get; set; }
            public String Id { get; set; }
            public String Status { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            cbDiscoverability.SelectedIndex = 2;
            cbPairingProcedure.SelectedIndex = 1;

            wclWiFiDirectAdvertiser = new wclWiFiDirectAdvertiser();
            wclWiFiDirectAdvertiser.OnAcceptDevice += new wclWiFiDirectAcceptDeviceEvent(wclWiFiDirectAdvertiser_OnAcceptDevice);
            wclWiFiDirectAdvertiser.OnStarted += new EventHandler(wclWiFiDirectAdvertiser_OnStarted);
            wclWiFiDirectAdvertiser.OnStopped += new EventHandler(wclWiFiDirectAdvertiser_OnStopped);
            wclWiFiDirectAdvertiser.OnPairProvidePin += new wclWiFiDirectPairProvidePinEvent(wclWiFiDirectAdvertiser_OnPairProvidePin);
            wclWiFiDirectAdvertiser.OnPairDisplayPin += new wclWiFiDirectPairDisplayPinEvent(wclWiFiDirectAdvertiser_OnPairDisplayPin);
            wclWiFiDirectAdvertiser.OnPairConfirm += new wclWiFiDirectPairConfirmEvent(wclWiFiDirectAdvertiser_OnPairConfirm);
            wclWiFiDirectAdvertiser.OnPairCompleted += new wclWiFiDirectPairCompletedEvent(wclWiFiDirectAdvertiser_OnPairCompleted);
            wclWiFiDirectAdvertiser.OnDeviceDisconnected += new wclWiFiDirectDisconnectedEvent(wclWiFiDirectAdvertiser_OnDeviceDisconnected);
            wclWiFiDirectAdvertiser.OnPairGetParams += new wclWiFiDirectPairGetParamsEvent(wclWiFiDirectAdvertiser_OnPairGetParams);
            wclWiFiDirectAdvertiser.OnDeviceConnected += new wclWiFiDirectConnectedEvent(wclWiFiDirectAdvertiser_OnDeviceConnected);
        }

        void wclWiFiDirectAdvertiser_OnStarted(object sender, EventArgs e)
        {
            lbLog.Items.Add("Advertiser has been started");
        }

        void wclWiFiDirectAdvertiser_OnStopped(object sender, EventArgs e)
        {
            lvDevices.Items.Clear();

            lbLog.Items.Add("Advertiser has been stopped");
        }

        void wclWiFiDirectAdvertiser_OnAcceptDevice(object Sender, wclWiFiDirectDevice Device, out bool Accept)
        {
            lbLog.Items.Add("Accepting device " + Device.Id);
            Accept = MessageBox.Show("Accept connection from " + Device.Name + "?", "Connection",
                MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes;
            if (Accept)
            {
                lvDevices.Items.Add(new DeviceItem
                {
                    DeviceName = Device.Name,
                    Id = Device.Id,
                    Status = "Connecting"
                });
            }
        }

        void wclWiFiDirectAdvertiser_OnDeviceDisconnected(object Sender, wclWiFiDirectDevice Device, int Reason)
        {
            DeviceItem Item = GetItem(Device.Id);
            if (Item != null)
            {
                lbLog.Items.Add(Device.Name + " disconnected. Reason: 0x" + Reason.ToString("X8"));
                lvDevices.Items.Remove(Item);
            }
        }

        void wclWiFiDirectAdvertiser_OnDeviceConnected(object Sender, wclWiFiDirectDevice Device, int Error)
        {
            DeviceItem Item = GetItem(Device.Id);
            if (Item != null)
            {
                if (Error == wclErrors.WCL_E_SUCCESS)
                {
                    Item.Status = "Connected";

                    lbLog.Items.Add(Device.Name + " connected");
                    lbLog.Items.Add("  Local IP: " + Device.LocalAddress);
                    lbLog.Items.Add("  Remote IP: " + Device.RemoteAddress);

                    lvDevices.Items.Refresh();
                }
                else
                {
                    lbLog.Items.Add(Device.Name + " connect error: 0x" + Error.ToString("X8"));
                    lvDevices.Items.Remove(Item);
                }
            }
        }

        void wclWiFiDirectAdvertiser_OnPairCompleted(object Sender, wclWiFiDirectDevice Device, int Result)
        {
            lbLog.Items.Add(Device.Name + " paired. Result: 0x" + Result.ToString("X8"));
        }

        void wclWiFiDirectAdvertiser_OnPairConfirm(object Sender, wclWiFiDirectDevice Device, out bool Confirm)
        {
            Confirm = MessageBox.Show("Confirm " + Device.Name + " connection", "Pairing",
                MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes;
        }

        void wclWiFiDirectAdvertiser_OnPairDisplayPin(object Sender, wclWiFiDirectDevice Device, string Pin)
        {
            MessageBox.Show("Use " + Pin + " PIN to pair with " + Device.Name);
        }

        void wclWiFiDirectAdvertiser_OnPairGetParams(object Sender, wclWiFiDirectDevice Device, out short GroupOwnerIntent, out wclWiFiDirectConfigurationMethod ConfigurationMethods, out wclWiFiDirectPairingProcedure PairingProcedure)
        {
            lbLog.Items.Add(Device.Name + " quering pairing params");

            switch (cbPairingProcedure.SelectedIndex)
            {
                case 0:
                    PairingProcedure = wclWiFiDirectPairingProcedure.ppInvitation;
                    break;
                case 1:
                    PairingProcedure = wclWiFiDirectPairingProcedure.ppGroupOwnerNegotiation;
                    break;
                default:
                    PairingProcedure = wclWiFiDirectPairingProcedure.ppGroupOwnerNegotiation;
                    break;
            }
            GroupOwnerIntent = Convert.ToInt16(edGroupOwnerIntent.Text);
            ConfigurationMethods = 0;
            if (cbDisplayPIN.IsChecked == true)
                ConfigurationMethods |= wclWiFiDirectConfigurationMethod.cmDisplayPin;
            if (cbProvidePIN.IsChecked == true)
                ConfigurationMethods |= wclWiFiDirectConfigurationMethod.cmProvidePin;
            if (cbPushButton.IsChecked == true)
                ConfigurationMethods |= wclWiFiDirectConfigurationMethod.cmPushButton;
        }

        void wclWiFiDirectAdvertiser_OnPairProvidePin(object Sender, wclWiFiDirectDevice Device, out string Pin)
        {
            Pin = Microsoft.VisualBasic.Interaction.InputBox("Enter PIN for ", "Pairing", "");
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            wclWiFiDirectAdvertiser.Stop();
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbLog.Items.Clear();
        }

        private void btDisconnect_Click(object sender, RoutedEventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                for (int i = 0; i < wclWiFiDirectAdvertiser.Count; i++)
                {
                    if (wclWiFiDirectAdvertiser[i].Id == ((DeviceItem)lvDevices.SelectedItems[0]).Id)
                    {
                        Int32 Res = wclWiFiDirectAdvertiser[i].Disconnect();
                        if (Res != wclErrors.WCL_E_SUCCESS)
                            MessageBox.Show("Disconnect error: 0x" + Res.ToString("X8"));
                        break;
                    }
                }
            }
        }

        private void btStop_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclWiFiDirectAdvertiser.Stop();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Stop error: 0x" + Res.ToString("X8"));
        }

        private void btStart_Click(object sender, RoutedEventArgs e)
        {
            if (wclWiFiDirectAdvertiser.Active)
                MessageBox.Show("Already running");
            else
            {
                switch (cbDiscoverability.SelectedIndex)
                {
                    case 0:
                        wclWiFiDirectAdvertiser.Discoverability = wclWiFiDirectAdvertisementDiscoverability.adIntensive;
                        break;
                    case 1:
                        wclWiFiDirectAdvertiser.Discoverability = wclWiFiDirectAdvertisementDiscoverability.adNone;
                        break;
                    case 2:
                        wclWiFiDirectAdvertiser.Discoverability = wclWiFiDirectAdvertisementDiscoverability.adNormal;
                        break;
                }
                wclWiFiDirectAdvertiser.AutonomousGroupOwnerEnabled = cbAutonomousGroupOwner.IsChecked == true;

                Int32 Res = wclWiFiDirectAdvertiser.Start();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Start error: 0x" + Res.ToString("X8"));
            }
        }

        private DeviceItem GetItem(String Id)
        {
            foreach (DeviceItem Item in lvDevices.Items)
                if (Item.Id == Id)
                    return Item;
            return null;
        }
    }
}
