﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclWiFi;
using wclCommon;

namespace WiFiSniffer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclWiFiClient Client;
        private wclWiFiSniffer Sniffer;

        internal class InterfaceItem
        {
            public String FriendlyName { get; set; }
            public String Id { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            cbPhy.SelectedIndex = 0;

            Sniffer = new wclWiFiSniffer();
            Client = new wclWiFiClient();

            Sniffer.AfterOpen += new EventHandler(Sniffer_AfterOpen);
            Sniffer.BeforeClose += new EventHandler(Sniffer_BeforeClose);
            Sniffer.OnRawFrameReceived += new wclWiFiSnifferRawFrameReceived(Sniffer_OnRawFrameReceived);
        }

        void Sniffer_AfterOpen(object sender, EventArgs e)
        {
            RefreshChannel();
            RefreshPhy();
            Int64 Mac;
            Int32 Res = Sniffer.GetMacAddr(out Mac);
            if (Res == wclErrors.WCL_E_SUCCESS)
            {
                laMacAddress.Content = Mac.ToString("X12");
                switch (Sniffer.Mode)
                {
                    case wclWiFiOperationMode.omUnknown:
                        laIfaceMode.Content = "Unknown";
                        break;
                    case wclWiFiOperationMode.omStation:
                        laIfaceMode.Content = "Station";
                        break;
                    case wclWiFiOperationMode.omAccessPoint:
                        laIfaceMode.Content = "Access point";
                        break;
                    case wclWiFiOperationMode.omExtensibleStation:
                        laIfaceMode.Content = "Extensible Station";
                        break;
                    case wclWiFiOperationMode.omExtensibleAccessPoint:
                        laIfaceMode.Content = "Extensible Access Point";
                        break;
                    case wclWiFiOperationMode.omWiFiDirectDevice:
                        laIfaceMode.Content = "WiFiDirect Device";
                        break;
                    case wclWiFiOperationMode.omWiFiDirectGroupOwner:
                        laIfaceMode.Content = "WiFiDirect Group Owner";
                        break;
                    case wclWiFiOperationMode.omWiFiDirectClient:
                        laIfaceMode.Content = "WiFiDirect Client";
                        break;
                    case wclWiFiOperationMode.omManufacturing:
                        laIfaceMode.Content = "Manufacturing";
                        break;
                    case wclWiFiOperationMode.omNetworkMonitor:
                        laIfaceMode.Content = "Network Monitor";
                        break;
                    default:
                        laIfaceMode.Content = "Unknown";
                        break;
                }
            }
            else
                laMacAddress.Content = "Error: 0x" + Res.ToString("X8");
        }

        void Sniffer_BeforeClose(object sender, EventArgs e)
        {
            laMacAddress.Content = "000000000000";
            laPhy.Content = "NONE";
            laChannel.Content = "0";
            laIfaceMode.Content = "Unknown";
        }

        void Sniffer_OnRawFrameReceived(object Sender, byte[] Buffer)
        {
            String s = "";
            for (int i = 0; i < Buffer.Length; i++)
            {
                s = s + Buffer[i].ToString("X2") + " ";
                if (s.Length == 96)
                {
                    lbFrames.Items.Add(s);
                    s = "";
                }
            }
            if (s.Length > 0)
                lbFrames.Items.Add(s);
        }

        private void RefreshChannel()
        {
            UInt32 Ch;
            Int32 Res = Sniffer.GetChannel(out Ch);
            if (Res == wclErrors.WCL_E_SUCCESS)
                laChannel.Content = Ch.ToString();
            else
                laChannel.Content = "Error: 0x" + Res.ToString("X8");
        }

        private void RefreshPhy()
        {
            wclWiFiSnifferPhy Phy;
            Int32 Res = Sniffer.GetPhy(out Phy);
            if (Res == wclErrors.WCL_E_SUCCESS)
            {
                switch (Phy)
                {
                    case wclWiFiSnifferPhy.ph802_11a:
                        laPhy.Content = "802.11a";
                        break;
                    case wclWiFiSnifferPhy.ph802_11b:
                        laPhy.Content = "802.11b";
                        break;
                    case wclWiFiSnifferPhy.ph802_11g:
                        laPhy.Content = "802.11g";
                        break;
                    case wclWiFiSnifferPhy.ph802_11n:
                        laPhy.Content = "802.11n";
                        break;
                    default:
                        laPhy.Content = "UNKNOWN";
                        break;
                }
            }
            else
                laPhy.Content = "Error: 0x" + Res.ToString("X8");
        }

        private void ShowError(String s, Int32 Error)
        {
            MessageBox.Show(s + ": 0x" + Error.ToString("X8"), "Error", MessageBoxButton.OK, MessageBoxImage.Error);
        }

        private void btEnumInterfaces_Click(object sender, RoutedEventArgs e)
        {
            lvInterfaces.Items.Clear();

            Int32 Res = Client.Open();
            if (Res != wclErrors.WCL_E_SUCCESS)
                ShowError("Unable to open WiFi Client", Res);
            else
            {
                try
                {
                    wclWiFiInterfaceData[] Ifaces;
                    Res = Client.EnumInterfaces(out Ifaces);
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        ShowError("Unable to enumerate WiFi Interfaces", Res);
                    else
                    {
                        try
                        {
                            if (Ifaces == null)
                                MessageBox.Show("No WiFi Interfaces were found", "Warning", MessageBoxButton.OK, MessageBoxImage.Warning);
                            else
                            {
                                if (Ifaces.Length == 0)
                                    MessageBox.Show("No WiFi Interfaces were found", "Warning", MessageBoxButton.OK, MessageBoxImage.Warning);
                                else
                                {
                                    for (int i = 0; i < Ifaces.Length; i++)
                                    {
                                        lvInterfaces.Items.Add(new InterfaceItem
                                        {
                                            FriendlyName = Ifaces[i].Description,
                                            Id = Ifaces[i].Id.ToString()
                                        });
                                    }
                                }
                            }
                        }
                        finally
                        {
                            Ifaces = null;
                        }
                    }
                }
                finally
                {
                    Client.Close();
                }
            }
        }

        private void btStartCapture_Click(object sender, RoutedEventArgs e)
        {
            if (lvInterfaces.SelectedItems.Count == 0)
                MessageBox.Show("Select interface");
            else
            {
                Guid Id = new Guid(((InterfaceItem)lvInterfaces.SelectedItems[0]).Id);
                try
                {
                    Sniffer.DoNotChangeMode = cbDoNotChangeMode.IsChecked.Value;
                Int32 Res = Sniffer.Open(Id);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    ShowError("Unable start capturing", Res);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void btStopCapture_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = Sniffer.Close();
            if (Res != wclErrors.WCL_E_SUCCESS)
                ShowError("Unable to stop capturing", Res);
        }

        private void btSetChannel_Click(object sender, RoutedEventArgs e)
        {
            UInt32 Ch = Convert.ToUInt32(edChannel.Text);
            Int32 Res = Sniffer.SetChannel(Ch);
            if (Res != wclErrors.WCL_E_SUCCESS)
                ShowError("Unable to change channel", Res);
            else
                RefreshChannel();
        }

        private void btSetPhy_Click(object sender, RoutedEventArgs e)
        {
            wclWiFiSnifferPhy Phy = (wclWiFiSnifferPhy)cbPhy.SelectedIndex;
            Int32 Res = Sniffer.SetPhy(Phy);
            if (Res != wclErrors.WCL_E_SUCCESS)
                ShowError("Unable to change PHY", Res);
            else
                RefreshPhy();
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbFrames.Items.Clear();
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            Sniffer.Close();

            Sniffer = null;
            Client = null;
        }
    }
}
