﻿using System;
using System.Windows.Forms;

using wclCommon;
using wclWiFi;

namespace MobileHotspot
{
    public partial class fmMain : Form
    {
        private wclMobileHotspot wclMobileHotspot;

        public fmMain()
        {
            InitializeComponent();
        }

        private void Trace(String Msg)
        {
            MessageBox.Show(Msg);
        }

        private void Trace(String Msg, Int32 Error)
        {
            Trace(Msg + ". Error: 0x" + Error.ToString("X8"));
        }

        private void fmMain_Load(object sender, EventArgs e)
        {
            wclMobileHotspot = new wclMobileHotspot();
            wclMobileHotspot.OnClosed += new EventHandler(wclMobileHotspot_OnClosed);
            wclMobileHotspot.OnOpened += new EventHandler(wclMobileHotspot_OnOpened);

            cbBand.SelectedIndex = 0;
        }

        void wclMobileHotspot_OnOpened(object sender, EventArgs e)
        {
            Trace("Mobile hotspot opened.");
            String Ssid;
            if (wclMobileHotspot.GetSsid(out Ssid) == wclErrors.WCL_E_SUCCESS)
                edSsid.Text = Ssid;
            String Passphrase;
            if (wclMobileHotspot.GetPassphrase(out Passphrase) == wclErrors.WCL_E_SUCCESS)
                edPassword.Text = Passphrase;
        }

        void wclMobileHotspot_OnClosed(object sender, EventArgs e)
        {
            Trace("Mobile hotspot closed.");

            edSsid.Text = "";
            edPassword.Text = "";
        }

        private void fmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            wclMobileHotspot.Close();
            wclMobileHotspot = null;
        }

        private void btOpen_Click(object sender, EventArgs e)
        {
            Int32 Res = wclMobileHotspot.Open();
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Open failed", Res);
        }

        private void btClose_Click(object sender, EventArgs e)
        {
            Int32 Res = wclMobileHotspot.Close();
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Close failed", Res);
        }

        private void btStart_Click(object sender, EventArgs e)
        {
            Int32 Res = wclMobileHotspot.Start();
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Start failed", Res);
        }

        private void btStop_Click(object sender, EventArgs e)
        {
            Int32 Res = wclMobileHotspot.Stop();
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Stop failed", Res);
        }

        private void btGetState_Click(object sender, EventArgs e)
        {
            wclMobileHotspotState State;
            Int32 Res = wclMobileHotspot.GetState(out State);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Get state failed", Res);
            else
            {
                switch (State)
                {
                    case wclMobileHotspotState.mhsUnknown:
                        Trace("State: unknown");
                        break;
                    case wclMobileHotspotState.mhsOn:
                        Trace("State: on");
                        break;
                    case wclMobileHotspotState.mhsOff:
                        Trace("State: off");
                        break;
                    case wclMobileHotspotState.mhsInTransition:
                        Trace("State: switching");
                        break;
                }
            }
        }

        private void btGetSsid_Click(object sender, EventArgs e)
        {
            String Ssid;
            Int32 Res = wclMobileHotspot.GetSsid(out Ssid);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Get SSID failed", Res);
            else
                edSsid.Text = Ssid;
        }

        private void btSetSsid_Click(object sender, EventArgs e)
        {
            Int32 Res = wclMobileHotspot.SetSsid(edSsid.Text);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Set SSID failed", Res);
        }

        private void btGetPwd_Click(object sender, EventArgs e)
        {
            String Pwd;
            Int32 Res = wclMobileHotspot.GetPassphrase(out Pwd);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Get passphrase failed", Res);
            else
                edPassword.Text = Pwd;
        }

        private void btSetPwd_Click(object sender, EventArgs e)
        {
            Int32 Res = wclMobileHotspot.SetPassphrase(edPassword.Text);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Set passphrase failed", Res);
        }

        private void btMaxClients_Click(object sender, EventArgs e)
        {
            UInt32 Count;
            Int32 Res = wclMobileHotspot.GetMaxClientCount(out Count);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Get max client connections failed", Res);
            else
                Trace("Maximum allowed connections: " + Count.ToString());
        }

        private void btConnected_Click(object sender, EventArgs e)
        {
            UInt32 Count;
            Int32 Res = wclMobileHotspot.GetClientCount(out Count);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Get connectioned clients count failed", Res);
            else
                Trace("Connected clients: " + Count.ToString());
        }

        private void btIsSupported_Click(object sender, EventArgs e)
        {
            wclMobileHotspotBand Band;
            switch (cbBand.SelectedIndex)
            {
                case 0:
                    Band = wclMobileHotspotBand.mhbAuto;
                    break;
                case 1:
                    Band = wclMobileHotspotBand.mhbTwoPointFourGigahertz;
                    break;
                default:
                    Band = wclMobileHotspotBand.mhbFiveGigahertz;
                    break;
            }

            Boolean Supp;
            Int32 Res = wclMobileHotspot.IsBandSupported(Band, out Supp);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Check band failed", Res);
            else
            {
                if (Supp)
                    MessageBox.Show("Band is supported");
                else
                    MessageBox.Show("Band is NOT supported");
            }
        }

        private void btSetBand_Click(object sender, EventArgs e)
        {
            wclMobileHotspotBand Band;
            switch (cbBand.SelectedIndex)
            {
                case 0:
                    Band = wclMobileHotspotBand.mhbAuto;
                    break;
                case 1:
                    Band = wclMobileHotspotBand.mhbTwoPointFourGigahertz;
                    break;
                default:
                    Band = wclMobileHotspotBand.mhbFiveGigahertz;
                    break;
            }

            Int32 Res = wclMobileHotspot.SetBand(Band);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Set band failed", Res);
        }

        private void btGetBand_Click(object sender, EventArgs e)
        {
            wclMobileHotspotBand Band;
            Int32 Res = wclMobileHotspot.GetBand(out Band);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Get band failed", Res);
            else
            {
                switch (Band)
                {
                    case wclMobileHotspotBand.mhbAuto:
                        MessageBox.Show("Band: Auto");
                        break;
                    case wclMobileHotspotBand.mhbTwoPointFourGigahertz:
                        MessageBox.Show("Band: 2.4");
                        break;
                    default:
                        MessageBox.Show("Band: 5.8");
                        break;
                }
            }
        }

        private void btConnectedClients_Click(object sender, EventArgs e)
        {
            lvConnectedClients.Items.Clear();
            wclMobileHotspotClient[] Clients;
            Int32 Res = wclMobileHotspot.GetClients(out Clients);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Get connected clients failed", Res);
            else
            {
                if (Clients != null && Clients.Length > 0)
                {
                    foreach (wclMobileHotspotClient Client in Clients)
                    {
                        ListViewItem Item = lvConnectedClients.Items.Add(Client.Mac);
                        Item.SubItems.Add(Client.Name);
                    }
                }
            }
        }

        private void btCheckTimeout_Click(object sender, EventArgs e)
        {
            Boolean Enabled;
            Int32 Res = wclMobileHotspot.IsNoConnectionsTimeoutEnabled(out Enabled);
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Get no connection timeout status failed", Res);
            else
            {
                if (Enabled)
                    MessageBox.Show("No connection timeout enabled");
                else
                    MessageBox.Show("No connection timeout disabled");
            }
        }

        private void btEnableTimeout_Click(object sender, EventArgs e)
        {
            Int32 Res = wclMobileHotspot.EnableNoConnectionsTimeout();
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Enable No Connection Timeout failed", Res);
        }

        private void btDisableTimeout_Click(object sender, EventArgs e)
        {
            Int32 Res = wclMobileHotspot.DisableNoConnectionsTimeout();
            if (Res != wclErrors.WCL_E_SUCCESS)
                Trace("Disable No Connection Timeout failed", Res);
        }
    }
}
