using System;
using System.Windows.Forms;

using wclWiFi;
using wclCommon;

namespace WiFiSniffer
{
    public partial class fmMain : Form
    {
        private wclWiFiClient Client;
        private wclWiFiSniffer Sniffer;

        public fmMain()
        {
            InitializeComponent();
        }

        private void fmMain_Load(object sender, EventArgs e)
        {
            cbPhy.SelectedIndex = 0;

            Sniffer = new wclWiFiSniffer();
            Client = new wclWiFiClient();

            Sniffer.AfterOpen += new EventHandler(Sniffer_AfterOpen);
            Sniffer.BeforeClose += new EventHandler(Sniffer_BeforeClose);
            Sniffer.OnRawFrameReceived += new wclWiFiSnifferRawFrameReceived(Sniffer_OnRawFrameReceived);
        }

        void Sniffer_OnRawFrameReceived(object Sender, byte[] Buffer)
        {
            String s = "";
            for (int i = 0; i < Buffer.Length; i++)
            {
                s = s + Buffer[i].ToString("X2") + " ";
                if (s.Length == 96)
                {
                    lbFrames.Items.Add(s);
                    s = "";
                }
            }
            if (s.Length > 0)
                lbFrames.Items.Add(s);

            Application.DoEvents();
        }

        void Sniffer_BeforeClose(object sender, EventArgs e)
        {
            laMacAddress.Text = "000000000000";
            laPhy.Text = "NONE";
            laChannel.Text = "0";
            laIfaceMode.Text = "Unknown";
        }

        void Sniffer_AfterOpen(object sender, EventArgs e)
        {
            RefreshChannel();
            RefreshPhy();
            Int64 Mac;
            Int32 Res = Sniffer.GetMacAddr(out Mac);
            if (Res == wclErrors.WCL_E_SUCCESS)
            {
                laMacAddress.Text = Mac.ToString("X12");
                switch (Sniffer.Mode)
                {
                    case wclWiFiOperationMode.omUnknown:
                        laIfaceMode.Text = "Unknown";
                        break;
                    case wclWiFiOperationMode.omStation:
                        laIfaceMode.Text = "Station";
                        break;
                    case wclWiFiOperationMode.omAccessPoint:
                        laIfaceMode.Text = "Access point";
                        break;
                    case wclWiFiOperationMode.omExtensibleStation:
                        laIfaceMode.Text = "Extensible Station";
                        break;
                    case wclWiFiOperationMode.omExtensibleAccessPoint:
                        laIfaceMode.Text = "Extensible Access Point";
                        break;
                    case wclWiFiOperationMode.omWiFiDirectDevice:
                        laIfaceMode.Text = "WiFiDirect Device";
                        break;
                    case wclWiFiOperationMode.omWiFiDirectGroupOwner:
                        laIfaceMode.Text = "WiFiDirect Group Owner";
                        break;
                    case wclWiFiOperationMode.omWiFiDirectClient:
                        laIfaceMode.Text = "WiFiDirect Client";
                        break;
                    case wclWiFiOperationMode.omManufacturing:
                        laIfaceMode.Text = "Manufacturing";
                        break;
                    case wclWiFiOperationMode.omNetworkMonitor:
                        laIfaceMode.Text = "Network Monitor";
                        break;
                    default:
                        laIfaceMode.Text = "Unknown";
                        break;
                }
            }
            else
                laMacAddress.Text = "Error: 0x" + Res.ToString("X8");
        }

        private void RefreshChannel()
        {
            UInt32 Ch;
            Int32 Res = Sniffer.GetChannel(out Ch);
            if (Res == wclErrors.WCL_E_SUCCESS)
                laChannel.Text = Ch.ToString();
            else
                laChannel.Text = "Error: 0x" + Res.ToString("X8");
        }

        private void RefreshPhy()
        {
            wclWiFiSnifferPhy Phy;
            Int32 Res = Sniffer.GetPhy(out Phy);
            if (Res == wclErrors.WCL_E_SUCCESS)
            {
                switch (Phy)
                {
                    case wclWiFiSnifferPhy.ph802_11a:
                        laPhy.Text = "802.11a";
                        break;
                    case wclWiFiSnifferPhy.ph802_11b:
                        laPhy.Text = "802.11b";
                        break;
                    case wclWiFiSnifferPhy.ph802_11g:
                        laPhy.Text = "802.11g";
                        break;
                    case wclWiFiSnifferPhy.ph802_11n:
                        laPhy.Text = "802.11n";
                        break;
                    default:
                        laPhy.Text = "UNKNOWN";
                        break;
                }
            }
            else
                laPhy.Text = "Error: 0x" + Res.ToString("X8");
        }

        private void ShowError(String s, Int32 Error)
        {
            MessageBox.Show(s + ": 0x" + Error.ToString("X8"), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void btEnumInterfaces_Click(object sender, EventArgs e)
        {
            lvInterfaces.Items.Clear();

            Int32 Res = Client.Open();
            if (Res != wclErrors.WCL_E_SUCCESS)
                ShowError("Unable to open WiFi Client", Res);
            else
            {
                try
                {
                    wclWiFiInterfaceData[] Ifaces;
                    Res = Client.EnumInterfaces(out Ifaces);
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        ShowError("Unable to enumerate WiFi Interfaces", Res);
                    else
                    {
                        try
                        {
                            if (Ifaces == null)
                                MessageBox.Show("No WiFi Interfaces were found", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            else
                            {
                                if (Ifaces.Length == 0)
                                    MessageBox.Show("No WiFi Interfaces were found", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                                else
                                {
                                    for (int i = 0; i < Ifaces.Length; i++)
                                    {
                                        ListViewItem Item = lvInterfaces.Items.Add(Ifaces[i].Description);
                                        Item.SubItems.Add(Ifaces[i].Id.ToString());
                                    }
                                }
                            }
                        }
                        finally
                        {
                            Ifaces = null;
                        }
                    }
                }
                finally
                {
                    Client.Close();
                }
            }
        }

        private void btStartCapture_Click(object sender, EventArgs e)
        {
            if (lvInterfaces.SelectedItems.Count == 0)
                MessageBox.Show("Select interface");
            else
            {
                Guid Id = new Guid(lvInterfaces.SelectedItems[0].SubItems[1].Text);
                try
                {
                    Sniffer.DoNotChangeMode = cbDoNotChangeMode.Checked;
                    Int32 Res = Sniffer.Open(Id);
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        ShowError("Unable start capturing", Res);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void btStopCapture_Click(object sender, EventArgs e)
        {
            Int32 Res = Sniffer.Close();
            if (Res != wclErrors.WCL_E_SUCCESS)
                ShowError("Unable to stop capturing", Res);
        }

        private void btSetChannel_Click(object sender, EventArgs e)
        {
            UInt32 Ch = Convert.ToUInt32(edChannel.Text);
            Int32 Res = Sniffer.SetChannel(Ch);
            if (Res != wclErrors.WCL_E_SUCCESS)
                ShowError("Unable to change channel", Res);
            else
                RefreshChannel();
        }

        private void btSetPhy_Click(object sender, EventArgs e)
        {
            wclWiFiSnifferPhy Phy = (wclWiFiSnifferPhy)cbPhy.SelectedIndex;
            Int32 Res = Sniffer.SetPhy(Phy);
            if (Res != wclErrors.WCL_E_SUCCESS)
                ShowError("Unable to change PHY", Res);
            else
                RefreshPhy();
        }

        private void btClear_Click(object sender, EventArgs e)
        {
            lbFrames.Items.Clear();
        }

        private void fmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            Sniffer.Close();

            Sniffer = null;
            Client = null;
        }
    }
}