Public Class fmMain
    Private WithEvents Watcher As wclWiFiDirectDeviceWatcher
    Private WithEvents Client As wclWiFiDirectClient

    Private Sub fmMain_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        Client.Disconnect()
        Watcher.Terminate()
    End Sub

    Private Sub fmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        cbPairingProcedure.SelectedIndex = 0

        Watcher = New wclWiFiDirectDeviceWatcher()
        Client = New wclWiFiDirectClient()
    End Sub

    Private Sub Client_OnDeviceConnected(ByVal Sender As Object, ByVal Device As wclWiFi.wclWiFiDirectDevice, ByVal [Error] As Integer) Handles Client.OnDeviceConnected
        lbLog.Items.Add("Connected with result: 0x" + [Error].ToString("X8"))
        If [Error] = wclErrors.WCL_E_SUCCESS Then
            lbLog.Items.Add("  Local IP: " + Client.LocalAddress)
            lbLog.Items.Add("  Remote IP: " + Client.RemoteAddress)
        End If
    End Sub

    Private Sub Client_OnDeviceDisconnected(ByVal Sender As Object, ByVal Device As wclWiFi.wclWiFiDirectDevice, ByVal Reason As Integer) Handles Client.OnDeviceDisconnected
        lbLog.Items.Add("Disconnected. Reason: 0x" + Reason.ToString("X8"))
    End Sub

    Private Sub Client_OnPairCompleted(ByVal Sender As Object, ByVal Device As wclWiFi.wclWiFiDirectDevice, ByVal Result As Integer) Handles Client.OnPairCompleted
        lbLog.Items.Add("Pairing completed with result: 0x" + Result.ToString("X8"))
    End Sub

    Private Sub Client_OnPairConfirm(ByVal Sender As Object, ByVal Device As wclWiFi.wclWiFiDirectDevice, ByRef Confirm As Boolean) Handles Client.OnPairConfirm
        Confirm = MessageBox.Show("Confirm pairing", "Pairing", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes
    End Sub

    Private Sub Client_OnPairDisplayPin(ByVal Sender As Object, ByVal Device As wclWiFi.wclWiFiDirectDevice, ByVal Pin As String) Handles Client.OnPairDisplayPin
        MessageBox.Show("Use PIN: " + Pin)
    End Sub

    Private Sub Client_OnPairGetParams(ByVal Sender As Object, ByVal Device As wclWiFi.wclWiFiDirectDevice, ByRef GroupOwnerIntent As Short, ByRef ConfigurationMethods As wclWiFi.wclWiFiDirectConfigurationMethod, ByRef PairingProcedure As wclWiFi.wclWiFiDirectPairingProcedure) Handles Client.OnPairGetParams
        GroupOwnerIntent = Convert.ToInt16(edGroupOwnerIntent.Text)

        ConfigurationMethods = 0
        If cbDisplayPIN.Checked Then
            ConfigurationMethods = ConfigurationMethods Or wclWiFiDirectConfigurationMethod.cmDisplayPin
        End If
        If cbProvidePIN.Checked Then
            ConfigurationMethods = ConfigurationMethods Or wclWiFiDirectConfigurationMethod.cmProvidePin
        End If
        If cbPushButton.Checked Then
            ConfigurationMethods = ConfigurationMethods Or wclWiFiDirectConfigurationMethod.cmPushButton
        End If

        If cbPairingProcedure.SelectedIndex = 0 Then
            PairingProcedure = wclWiFiDirectPairingProcedure.ppInvitation
        Else
            PairingProcedure = wclWiFiDirectPairingProcedure.ppGroupOwnerNegotiation
        End If
    End Sub

    Private Sub Client_OnPairProvidePin(ByVal Sender As Object, ByVal Device As wclWiFi.wclWiFiDirectDevice, ByRef Pin As String) Handles Client.OnPairProvidePin
        Pin = Microsoft.VisualBasic.Interaction.InputBox("Enter PIN for ", "Pairing", "", Left, Top)
    End Sub

    Private Sub Watcher_OnDeviceFound(ByVal Sender As Object, ByVal Id As String, ByVal Name As String) Handles Watcher.OnDeviceFound
        Dim Item As ListViewItem = lvDevices.Items.Add(Name)
        Item.SubItems.Add(Id)

        lbLog.Items.Add("Device found")
    End Sub

    Private Sub Watcher_OnDiscoveringCompleted(ByVal Sender As Object, ByVal [Error] As Integer) Handles Watcher.OnDiscoveringCompleted
        lbLog.Items.Add("Discovering completed with result: 0x" + [Error].ToString("X8"))
    End Sub

    Private Sub Watcher_OnDiscoveringStarted(ByVal sender As Object, ByVal e As System.EventArgs) Handles Watcher.OnDiscoveringStarted
        lvDevices.Items.Clear()
        lbLog.Items.Add("Discovering started")
    End Sub

    Private Sub btDiscover_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btDiscover.Click
        Dim Res As Int32 = Watcher.Discover()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Discover error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btTerminate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btTerminate.Click
        Dim Res As Int32 = Watcher.Terminate()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Terminate error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClear.Click
        lbLog.Items.Clear()
    End Sub

    Private Sub btDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btDisconnect.Click
        Dim Res As Int32 = Client.Disconnect()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Disconnect error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btConnect.Click
        If lvDevices.SelectedItems.Count = 0 Then
            MessageBox.Show("Select device")
        Else
            Dim Res As Int32 = Client.Connect(lvDevices.SelectedItems(0).SubItems(1).Text)
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Connect Error: 0x" + Res.ToString("X8"))
            End If
        End If
    End Sub

    Private Sub btIsPaired_Click(sender As System.Object, e As System.EventArgs) Handles btIsPaired.Click
        If lvDevices.SelectedItems.Count = 0 Then
            MessageBox.Show("Select device")
        Else
            Dim IsPaired As Boolean = False
            Dim Res As Integer = Watcher.IsPaired(lvDevices.SelectedItems(0).SubItems(1).Text, IsPaired)
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Get paired error: 0x" + Res.ToString("X8"))
            Else
                If IsPaired Then
                    MessageBox.Show("Device is paired")
                Else
                    MessageBox.Show("Device is NOT paired")
                End If
            End If
        End If
    End Sub

    Private Sub btUnpair_Click(sender As System.Object, e As System.EventArgs) Handles btUnpair.Click
        If lvDevices.SelectedItems.Count = 0 Then
            MessageBox.Show("Select device")
        Else
            Dim Res As Integer = Watcher.Unpair(lvDevices.SelectedItems(0).SubItems(1).Text)
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Unpair error: 0x" + Res.ToString("X8"))
            Else
                MessageBox.Show("Unpaired")
            End If
        End If
    End Sub

    Private Sub btGetName_Click(sender As System.Object, e As System.EventArgs) Handles btGetName.Click
        If lvDevices.SelectedItems.Count = 0 Then
            MessageBox.Show("Select device")
        Else
            Dim Name As String = ""
            Dim Res As Integer = Watcher.GetName(lvDevices.SelectedItems(0).SubItems(1).Text, Name)
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Get name error: 0x" + Res.ToString("X8"))
            Else
                MessageBox.Show("Device name: " + Name)
            End If
        End If
    End Sub

    Private Sub btEnumPaired_Click(sender As System.Object, e As System.EventArgs) Handles btEnumPaired.Click
        Dim Paired As List(Of String) = New List(Of String)()
        Dim Res As Int32 = Watcher.EnumPairedDevices(Paired)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Enum paired error: 0x" + Res.ToString("X8"))
        Else
            lvDevices.Items.Clear()

            If Paired.Count > 0 Then
                For Each Id As String In Paired
                    Dim Name As String = ""
                    Res = Watcher.GetName(Id, Name)
                    If Res <> wclErrors.WCL_E_SUCCESS Then
                        Name = "Error: 0x" + Res.ToString("X8")
                    End If

                    Dim Item As ListViewItem = lvDevices.Items.Add(Name)
                    Item.SubItems.Add(Id)
                Next
            End If
        End If
    End Sub
End Class
