Public Class fmMain
    Private WithEvents Sniffer As wclWiFiSniffer
    Private Client As wclWiFiClient

    Private Sub fmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        cbPhy.SelectedIndex = 0

        Sniffer = New wclWiFiSniffer()
        Client = New wclWiFiClient()
    End Sub

    Private Sub Sniffer_AfterOpen(ByVal sender As Object, ByVal e As System.EventArgs) Handles Sniffer.AfterOpen
        RefreshChannel()
        RefreshPhy()
        Dim Mac As Int64
        Dim Res As Int32 = Sniffer.GetMacAddr(Mac)
        If Res = wclErrors.WCL_E_SUCCESS Then
            laMacAddress.Text = Mac.ToString("X12")
            Select Sniffer.Mode
                Case wclWiFiOperationMode.omUnknown
                    laIfaceMode.Text = "Unknown"
                Case wclWiFiOperationMode.omStation
                    laIfaceMode.Text = "Station"
                Case wclWiFiOperationMode.omAccessPoint
                    laIfaceMode.Text = "Access point"
                Case wclWiFiOperationMode.omExtensibleStation
                    laIfaceMode.Text = "Extensible Station"
                Case wclWiFiOperationMode.omExtensibleAccessPoint
                    laIfaceMode.Text = "Extensible Access Point"
                Case wclWiFiOperationMode.omWiFiDirectDevice
                    laIfaceMode.Text = "WiFiDirect Device"
                Case wclWiFiOperationMode.omWiFiDirectGroupOwner
                    laIfaceMode.Text = "WiFiDirect Group Owner"
                Case wclWiFiOperationMode.omWiFiDirectClient
                    laIfaceMode.Text = "WiFiDirect Client"
                Case wclWiFiOperationMode.omManufacturing
                    laIfaceMode.Text = "Manufacturing"
                Case wclWiFiOperationMode.omNetworkMonitor
                    laIfaceMode.Text = "Network Monitor"
                Case Else
                    laIfaceMode.Text = "Unknown"
            End Select
        Else
            laMacAddress.Text = "Error: 0x" + Res.ToString("X8")
        End If
    End Sub

    Private Sub Sniffer_BeforeClose(ByVal sender As Object, ByVal e As System.EventArgs) Handles Sniffer.BeforeClose
        laMacAddress.Text = "000000000000"
        laPhy.Text = "NONE"
        laChannel.Text = "0"
        laIfaceMode.Text = "Unknown"
    End Sub

    Private Sub Sniffer_OnRawFrameReceived(ByVal Sender As Object, ByVal Buffer() As Byte) Handles Sniffer.OnRawFrameReceived
        Dim s As String = ""
        Dim I As Integer
        For I = 0 To Buffer.Length - 1
            s = s + Buffer(I).ToString("X2") + " "
            If s.Length = 96 Then
                lbFrames.Items.Add(s)
                s = ""
            End If
                Next I
        If s.Length > 0 Then
            lbFrames.Items.Add(s)
        End If

        Application.DoEvents()
    End Sub

    Private Sub RefreshChannel()
        Dim Ch As UInt32
        Dim Res As Int32 = Sniffer.GetChannel(Ch)
        If Res = wclErrors.WCL_E_SUCCESS Then
            laChannel.Text = Ch.ToString()
        Else
            laChannel.Text = "Error: 0x" + Res.ToString("X8")
        End If
    End Sub

    Private Sub RefreshPhy()
        Dim Phy As wclWiFiSnifferPhy
        Dim Res As Int32 = Sniffer.GetPhy(Phy)
        If Res = wclErrors.WCL_E_SUCCESS Then
            Select Case Phy
                Case wclWiFiSnifferPhy.ph802_11a
                    laPhy.Text = "802.11a"
                Case wclWiFiSnifferPhy.ph802_11b
                    laPhy.Text = "802.11b"
                Case wclWiFiSnifferPhy.ph802_11g
                    laPhy.Text = "802.11g"
                Case wclWiFiSnifferPhy.ph802_11n
                    laPhy.Text = "802.11n"
                Case Else
                    laPhy.Text = "UNKNOWN"
            End Select
        Else
            laPhy.Text = "Error: 0x" + Res.ToString("X8")
        End If
    End Sub

    Private Sub ShowError(ByVal s As String, ByVal Err As Int32)
        MessageBox.Show(s + ": 0x" + Err.ToString("X8"), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Sub

    Private Sub btEnumInterfaces_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btEnumInterfaces.Click
        lvInterfaces.Items.Clear()

        Dim Res As Int32 = Client.Open()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            ShowError("Unable to open WiFi Client", Res)
        Else
            Try
                Dim Ifaces As wclWiFiInterfaceData() = Nothing
                Res = Client.EnumInterfaces(Ifaces)
                If Res <> wclErrors.WCL_E_SUCCESS Then
                    ShowError("Unable to enumerate WiFi Interfaces", Res)
                Else
                    Try
                        If Ifaces Is Nothing Then
                            MessageBox.Show("No WiFi Interfaces were found", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
                        Else
                            If Ifaces.Length = 0 Then
                                MessageBox.Show("No WiFi Interfaces were found", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
                            Else
                                Dim I As Int32
                                For I = 0 To Ifaces.Length - 1
                                    Dim Item As ListViewItem = lvInterfaces.Items.Add(Ifaces(I).Description)
                                    Item.SubItems.Add(Ifaces(I).Id.ToString())
                                Next I
                            End If
                        End If
                    Finally
                        Ifaces = Nothing
                    End Try
                End If
            Finally
                Client.Close()
            End Try
        End If
    End Sub

    Private Sub btStartCapture_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btStartCapture.Click
        If lvInterfaces.SelectedItems.Count = 0 Then
            MessageBox.Show("Select interface")
        Else
            Dim Id As Guid = New Guid(lvInterfaces.SelectedItems(0).SubItems(1).Text)
            Try
                Sniffer.DoNotChangeMode = cbDoNotChangeMode.Checked
                Dim Res As Int32 = Sniffer.Open(Id)
                If Res <> wclErrors.WCL_E_SUCCESS Then
                    ShowError("Unable start capturing", Res)
                End If
            Catch ex As Exception
                MessageBox.Show(ex.Message)
            End Try
        End If
    End Sub

    Private Sub btStopCapture_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btStopCapture.Click
        Dim Res As Int32 = Sniffer.Close()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            ShowError("Unable to stop capturing", Res)
        End If
    End Sub

    Private Sub btSetChannel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btSetChannel.Click
        Dim Ch As UInt32 = Convert.ToUInt32(edChannel.Text)
        Dim Res As Int32 = Sniffer.SetChannel(Ch)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            ShowError("Unable to change channel", Res)
        Else
            RefreshChannel()
        End If
    End Sub

    Private Sub btSetPhy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btSetPhy.Click
        Dim Phy As wclWiFiSnifferPhy = cbPhy.SelectedIndex
        Dim Res As Int32 = Sniffer.SetPhy(Phy)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            ShowError("Unable to change PHY", Res)
        Else
            RefreshPhy()
        End If
    End Sub

    Private Sub btClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClear.Click
        lbFrames.Items.Clear()
    End Sub

    Private Sub fmMain_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        Sniffer.Close()

        Sniffer = Nothing
        Client = Nothing
    End Sub
End Class
