﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using Microsoft.Win32;

using wclWiFi;
using wclCommon;

namespace WiFiClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclWiFiClient Client;
        private wclWiFiProfilesManager ProfilesManager;

        internal class InterfaceItem
        {
            public String InterfaceId { get; set; }
            public String Description { get; set; }
            public String Primary { get; set; }
        };

        internal class NetworkItem
        {
            public String ProfileName { get; set; }
            public String Ssid { get; set; }
            public String BssType { get; set; }
            public String NumberOfBss { get; set; }
            public String Connectable { get; set; }
            public String NotConnectableReason { get; set; }
            public String PhyTypes { get; set; }
            public String MorePhys { get; set; }
            public String SignalQuality { get; set; }
            public String SecurityEnabled { get; set; }
            public String AuthAlg { get; set; }
            public String CypherAlg { get; set; }
            public String Flags { get; set; }
        };

        internal class BssItem
        {
            public String Ssid { get; set; }
            public String PhyId { get; set; }
            public String Mac { get; set; }
            public String BssType { get; set; }
            public String PhyType { get; set; }
            public String Rssi { get; set; }
            public String LinkQuality { get; set; }
            public String InRegDomain { get; set; }
            public String BeaconPeriod { get; set; }
            public String Timestamp { get; set; }
            public String HostTimestamp { get; set; }
            public String Capability { get; set; }
            public String Freq { get; set; }
            public String Ies { get; set; }
        };

        internal class ProfileItem
        {
            public String Name { get; set; }
            public String Flags { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            cbClientBssType.SelectedIndex = 2;
            cbClientConnect.SelectedIndex = 0;

            Client = new wclWiFiClient();
            Client.AfterOpen += new EventHandler(Client_AfterOpen);
            Client.BeforeClose += new EventHandler(Client_BeforeClose);

            ProfilesManager = new wclWiFiProfilesManager();
        }

        void Client_AfterOpen(object sender, EventArgs e)
        {
            MessageBox.Show("WiFi client opened");

            ProfilesManager.Open();
        }

        void Client_BeforeClose(object sender, EventArgs e)
        {
            ProfilesManager.Close();

            ClientClearInterfaces();
            ClientClearNetworks();
            ClientClearBss();
            ClientClearProfiles();

            MessageBox.Show("WiFi client is closing");
        }

        private Boolean ShowResult(Int32 Res)
        {
            if (Res == wclErrors.WCL_E_SUCCESS)
                return true;

            MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            return false;
        }

        private void btOpen_Click(object sender, RoutedEventArgs e)
        {
            ShowResult(Client.Open());
        }

        private void btClose_Click(object sender, RoutedEventArgs e)
        {
            ShowResult(Client.Close());
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            Client.Close();
        }

        private void ClientClearInterfaces()
        {
            lvClientInterfaces.Items.Clear();
        }

        private void btEnumInterfaces_Click(object sender, RoutedEventArgs e)
        {
            wclWiFiInterfaceData[] Interfaces = null;
            ClientClearInterfaces();

            if (ShowResult(Client.EnumInterfaces(out Interfaces)))
            {
                if (Interfaces != null)
                {
                    for (Int32 i = 0; i < Interfaces.Length; i++)
                    {
                        lvClientInterfaces.Items.Add(new InterfaceItem
                        {
                            InterfaceId = Interfaces[i].Id.ToString(),
                            Description = Interfaces[i].Description,
                            Primary = Interfaces[i].Primary.ToString()
                        });
                    }
                }
            }
        }

        private Boolean ClientGetInterfaceId(out Guid Id)
        {
            Id = Guid.Empty;

            if (lvClientInterfaces.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select interface");
                return false;
            }

            Id = new Guid(((InterfaceItem)lvClientInterfaces.SelectedItems[0]).InterfaceId);
            return true;
        }

        private void ClientClearNetworks()
        {
            lvClientNetworks.Items.Clear();
        }

        private void ClientClearBss()
        {
            lvClientBss.Items.Clear();
        }

        private void ClientClearProfiles()
        {
            lvClientProfiles.Items.Clear();
        }

        private void btClientEnumNetworks_Click(object sender, RoutedEventArgs e)
        {
            Guid Id = Guid.Empty;

            ClientClearNetworks();
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiAvailableNetworkFilter Filters = 0;
                if (cbClientAdHocProfiles.IsChecked == true)
                    Filters |= wclWiFiAvailableNetworkFilter.ffIncludeAllAdhocProfiles;
                if (cbClientHiddenProfiles.IsChecked == true)
                    Filters |= wclWiFiAvailableNetworkFilter.ffIncludeAllManualHiddenProfiles;

                wclWiFiAvailableNetwork[] Networks = null;
                if (ShowResult(Client.EnumAvailableNetworks(Id, Filters, out Networks)))
                {
                    if (Networks != null)
                    {
                        for (Int32 i = 0; i < Networks.Length; i++)
                        {
                            lvClientNetworks.Items.Add(new NetworkItem
                            {
                                ProfileName = Networks[i].ProfileName,
                                Ssid = Networks[i].Ssid,
                                BssType = Networks[i].BssType.ToString(),
                                NumberOfBss = Networks[i].NumberOfBssids.ToString(),
                                Connectable = Networks[i].NetworkConnectable.ToString(),
                                NotConnectableReason = Networks[i].NotConnectableReason.ToString(),
                                PhyTypes = Networks[i].PhyTypes.ToString(),
                                MorePhys = Networks[i].MorePhyTypes.ToString(),
                                SignalQuality = Networks[i].SignalQuality.ToString(),
                                SecurityEnabled = Networks[i].SecurityEnabled.ToString(),
                                AuthAlg = Networks[i].DefaultAuthAlgorithm.ToString(),
                                CypherAlg = Networks[i].DefaultCipherAlgorithm.ToString(),
                                Flags = Networks[i].Flags.ToString()
                            });
                        }
                    }
                }
            }
        }

        private void btClientScan_Click(object sender, RoutedEventArgs e)
        {
            Guid Id = Guid.Empty;
            if (ClientGetInterfaceId(out Id))
            {
                if (ShowResult(Client.Scan(Id, edClientScanSsid.Text)))
                    MessageBox.Show("Scan started. Use EventsDemo to see scan events.");
            }
        }

        private void btClientEnumBss_Click(object sender, RoutedEventArgs e)
        {
            ClientClearBss();
            Guid Id = Guid.Empty;

            if (ClientGetInterfaceId(out Id))
            {
                String Ssid = "";

                if (cbClientUseSsid.IsChecked == true)
                {
                    if (lvClientNetworks.SelectedItems.Count == 0)
                    {
                        MessageBox.Show("Select network");
                        return;
                    }

                    Ssid = ((NetworkItem)lvClientNetworks.SelectedItems[0]).Ssid;
                }

                wclWiFiBssType BssType = (wclWiFiBssType)cbClientBssType.SelectedIndex;
                wclWiFiBss[] BssList = null;

                if (ShowResult(Client.EnumBss(Id, Ssid, BssType, cbClientSecurityEnabled.IsChecked == true, out BssList)))
                {
                    if (BssList != null)
                    {
                        for (Int32 i = 0; i < BssList.Length; i++)
                        {
                            String s = "";
                            if (BssList[i].IeRaw != null && BssList[i].IeRaw.Length > 0)
                            {
                                for (Int32 x = 0; x < BssList[i].IeRaw.Length; x++)
                                    s = s + BssList[i].IeRaw[x].ToString("X2");
                            }

                            lvClientBss.Items.Add(new BssItem
                            {
                                Ssid = BssList[i].Ssid,
                                PhyId = BssList[i].PhyId.ToString(),
                                Mac = BssList[i].Mac,
                                BssType = BssList[i].BssType.ToString(),
                                PhyType = BssList[i].PhyType.ToString(),
                                Rssi = BssList[i].Rssi.ToString(),
                                LinkQuality = BssList[i].LinkQuality.ToString(),
                                InRegDomain = BssList[i].InRegDomain.ToString(),
                                BeaconPeriod = BssList[i].BeaconPeriod.ToString(),
                                Timestamp = BssList[i].Timestamp.ToString(),
                                HostTimestamp = BssList[i].HostTimestamp.ToString(),
                                Capability = BssList[i].Capability.ToString(),
                                Freq = BssList[i].ChCenterFrequency.ToString(),
                                Ies = s
                            });
                        }
                    }
                }
            }
        }

        private void btClientDisconnect_Click(object sender, RoutedEventArgs e)
        {
            Guid Id = Guid.Empty;
            if (ClientGetInterfaceId(out Id))
                ShowResult(Client.Disconnect(Id));
        }

        private void btClientGetShowDeniedNetworks_Click(object sender, RoutedEventArgs e)
        {
            Boolean Enabled = false;
            if (ShowResult(Client.AcmGetShowDeniedNetworks(out Enabled)))
            {
                if (Enabled)
                    MessageBox.Show("Show denied networks");
                else
                    MessageBox.Show("Do not show denied networks");
            }
        }

        private void btClientSetShowDeniedNetworks_Click(object sender, RoutedEventArgs e)
        {
            MessageBoxResult Res = MessageBox.Show("Show denied networks?", "Show denied networks",
                MessageBoxButton.YesNoCancel, MessageBoxImage.Question);
            if (Res == MessageBoxResult.Cancel)
                return;
            ShowResult(Client.AcmSetShowDeniedNetworks(Res == MessageBoxResult.Yes));
        }

        private void btClientGetPowerSettings_Click(object sender, RoutedEventArgs e)
        {
            wclWiFiPowerSetting Setting;
            if (ShowResult(Client.AcmGetPowerSetting(out Setting)))
                MessageBox.Show(Setting.ToString());
        }

        private void btClientGetOnlyUseGpProfiles_Click(object sender, RoutedEventArgs e)
        {
            Boolean Enabled;
            if (ShowResult(Client.AcmGetOnlyUseGpProfiles(out Enabled)))
            {
                if (Enabled)
                    MessageBox.Show("Only use GP profiles ENABLED");
                else
                    MessageBox.Show("Only use GP profiles DISABLED");
            }
        }

        private void btClientGetAllowExplicitCreds_Click(object sender, RoutedEventArgs e)
        {
            Boolean Enabled;
            if (ShowResult(Client.AcmGetAllowExplicitCreds(out Enabled)))
            {
                if (Enabled)
                    MessageBox.Show("Explicit creds ALLOWED");
                else
                    MessageBox.Show("Explicit creds NOT ALLOWED");
            }
        }

        private void btClientSetAllowExplicitCreds_Click(object sender, RoutedEventArgs e)
        {
            MessageBoxResult Res = MessageBox.Show("Allow explicit creds?", "Allow explicit creds?",
                MessageBoxButton.YesNoCancel, MessageBoxImage.Question);
            if (Res == MessageBoxResult.Cancel)
                return;
            ShowResult(Client.AcmSetAllowExplicitCreds(Res == MessageBoxResult.Yes));
        }

        private void btClientGetBlockPeriod_Click(object sender, RoutedEventArgs e)
        {
            UInt32 Period;
            if (ShowResult(Client.AcmGetBlockPeriod(out Period)))
                MessageBox.Show("Block period: " + Period.ToString());
        }

        private void btClientSetBlockPeriod_Click(object sender, RoutedEventArgs e)
        {
            // SET IT TO 10 here
            UInt32 Period = 10;
            ShowResult(Client.AcmSetBlockPeriod(Period));
        }

        private void btClientGetVirtualStationExtensibility_Click(object sender, RoutedEventArgs e)
        {
            Boolean Allowed;
            if (ShowResult(Client.AcmGetVirtualStationExtensibility(out Allowed)))
            {
                if (Allowed)
                    MessageBox.Show("Virtual station extensibility ALLOWED");
                else
                    MessageBox.Show("Virtual station extensibility NOT ALLOWED");
            }
        }

        private void btClientSetVirtualStationExtensibility_Click(object sender, RoutedEventArgs e)
        {
            MessageBoxResult Res = MessageBox.Show("Allow wirtual station extensibility?",
                "Allow wirtual station extensibility?", MessageBoxButton.YesNoCancel, MessageBoxImage.Question);
            if (Res == MessageBoxResult.Cancel)
                return;

            ShowResult(Client.AcmSetVirtualStationExtensibility(Res == MessageBoxResult.Yes));
        }

        private void btClientGetInterfaceState_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    try
                    {
                        wclWiFiInterfaceState State;
                        if (ShowResult(Iface.GetState(out State)))
                            MessageBox.Show("Interface state: " + State.ToString());
                    }
                    finally
                    {
                        Iface.Close();
                    }
                }
            }
        }

        private void btClientEnumProfiles_Click(object sender, RoutedEventArgs e)
        {
            ClientEnumProfiles();
        }

        private void btClientDeleteProfile_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                {
                    if (ShowResult(ProfilesManager.DeleteProfile(Id, Profile)))
                        ClientEnumProfiles();
                }
            }
        }

        private void ClientEnumProfiles()
        {
            ClientClearProfiles();

            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiProfileData[] Profiles;
                if (ShowResult(ProfilesManager.GetProfileList(Id, out Profiles)))
                {
                    if (Profiles != null)
                    {
                        for (Int32 i = 0; i < Profiles.Length; i++)
                        {
                            lvClientProfiles.Items.Add(new ProfileItem
                            {
                                Name = Profiles[i].Name,
                                Flags = Profiles[i].Flags.ToString()
                            });
                        }
                    }
                }
            }
        }

        private Boolean ClientGetProfile(out String Name)
        {
            Name = "";

            if (lvClientProfiles.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select profile:");
                return false;
            }

            Name = ((ProfileItem)lvClientProfiles.SelectedItems[0]).Name;

            return true;
        }

        private void btClientRenameProfile_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                {
                    // Use New_<OLD_PROFILE_NAME>
                    if (ShowResult(ProfilesManager.RenameProfile(Id, Profile, "NEW_" + Profile)))
                        ClientEnumProfiles();
                }
            }
        }

        private void btClientGetprofileXml_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                {
                    wclWiFiProfileFlag Flags = 0;
                    if (cbPlainText.IsChecked.Value)
                        Flags |= wclWiFiProfileFlag.pfGetPlaintextKey;
                    String Xml;
                    if (ShowResult(ProfilesManager.GetProfile(Id, Profile, ref Flags, out Xml)))
                    {
                        SaveFileDialog saveFileDialog = new SaveFileDialog();
                        saveFileDialog.FileName = Profile + ".xml";
                        if (saveFileDialog.ShowDialog() == true)
                            System.IO.File.WriteAllText(saveFileDialog.FileName, Xml);
                    }
                }
            }
        }

        private void btClientSetProfileXml_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                OpenFileDialog openFileDialog = new OpenFileDialog();
                openFileDialog.FileName = "";
                if (openFileDialog.ShowDialog() == true)
                {
                    String Xml;
                    Xml = System.IO.File.ReadAllText(openFileDialog.FileName);
                    ShowResult(ProfilesManager.SetProfile(Id, 0, Xml, true));
                }
            }
        }

        private void btClientConnect_Click(object sender, RoutedEventArgs e)
        {
            switch (cbClientConnect.SelectedIndex)
            {
                case 0:
                    ClientConnectUsingNetworkProfile();
                    break;
                case 1:
                    ClientConnectUsingSelectedprofile();
                    break;
                case 2:
                    ClientConnectUsingSelectedNetwork();
                    break;
                case 3:
                    ClientConenctUsingTemporaryProfile();
                    break;
            }
        }

        private void ClientConenctUsingTemporaryProfile()
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                if (lvClientNetworks.SelectedItems.Count == 0)
                    MessageBox.Show("Select network");
                else
                {
                    if (cbUseBssMac.IsChecked == true && lvClientBss.SelectedItems.Count == 0)
                        MessageBox.Show("Select Bss");
                    else
                    {
                        String Mac = "";
                        if (cbUseBssMac.IsChecked == true)
                            Mac = ((BssItem)lvClientBss.SelectedItems[0]).Mac;

                        String Ssid = ((NetworkItem)lvClientNetworks.SelectedItems[0]).Ssid;

                        Byte[] Utf8 = Encoding.UTF8.GetBytes(Ssid);
                        String Hex = "";
                        foreach (Byte b in Utf8)
                            Hex = Hex + b.ToString("X2");

                        String AuthAlg;
                        if (((NetworkItem)lvClientNetworks.SelectedItems[0]).AuthAlg == "auOpen")
                            AuthAlg = "open";
                        else
                        {
                            if (((NetworkItem)lvClientNetworks.SelectedItems[0]).AuthAlg == "auSharedKey")
                                AuthAlg = "shared";
                            else
                            {
                                if (((NetworkItem)lvClientNetworks.SelectedItems[0]).AuthAlg == "auWpa")
                                    AuthAlg = "WPA";
                                else
                                {
                                    if (((NetworkItem)lvClientNetworks.SelectedItems[0]).AuthAlg == "auWpaPsk")
                                        AuthAlg = "WPAPSK";
                                    else
                                    {
                                        if (((NetworkItem)lvClientNetworks.SelectedItems[0]).AuthAlg == "auRsna")
                                            AuthAlg = "WPA2";
                                        else
                                        {
                                            if (((NetworkItem)lvClientNetworks.SelectedItems[0]).AuthAlg == "auWpa3")
                                                AuthAlg = "WPA3ENT192";
                                            else
                                            {
                                                if (((NetworkItem)lvClientNetworks.SelectedItems[0]).AuthAlg == "auWpa3Sae")
                                                    AuthAlg = "WPA3SAE";
                                                else
                                                {
                                                    if (((NetworkItem)lvClientNetworks.SelectedItems[0]).AuthAlg == "auWpa3Ent")
                                                        AuthAlg = "WPA3ENT";
                                                    else
                                                    {
                                                        if (((NetworkItem)lvClientNetworks.SelectedItems[0]).AuthAlg == "auOwe")
                                                            AuthAlg = "OWE";
                                                        else
                                                            AuthAlg = "WPA2PSK";
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        String CipherAlg;
                        if (((NetworkItem)lvClientNetworks.SelectedItems[0]).CypherAlg == "caNone")
                            CipherAlg = "none";
                        else
                        {
                            if (((NetworkItem)lvClientNetworks.SelectedItems[0]).CypherAlg == "caTkip")
                                CipherAlg = "TKIP";
                            else
                            {
                                if (((NetworkItem)lvClientNetworks.SelectedItems[0]).CypherAlg == "caWep40" ||
                                    ((NetworkItem)lvClientNetworks.SelectedItems[0]).CypherAlg == "caWep104" ||
                                    ((NetworkItem)lvClientNetworks.SelectedItems[0]).CypherAlg == "caWep")
                                {
                                    CipherAlg = "WEP";
                                }
                                else
                                {
                                    if (((NetworkItem)lvClientNetworks.SelectedItems[0]).CypherAlg == "caGcmp256")
                                        CipherAlg = "GCMP256";
                                    else
                                        CipherAlg = "AES";
                                }
                            }
                        }

                        String Xml = "<?xml version='1.0'?>" +
                            "<WLANProfile xmlns='http://www.microsoft.com/networking/WLAN/profile/v1'>" +
                            "<name>" + Ssid + "</name>" +
                            "<SSIDConfig>" +
                            "<SSID>" +
                            "<hex>" + Hex + "</hex>" +
                            "<name>" + Ssid + "</name>" +
                            "</SSID>" +
                            "</SSIDConfig>" +
                            "<connectionType>ESS</connectionType>" +
                            "<connectionMode>manual</connectionMode>" +
                            "<autoSwitch>false</autoSwitch>" +
                            "<MSM>" +
                            "<security>" +
                            "<authEncryption>" +
                            "<authentication>" + AuthAlg + "</authentication>" +
                            "<encryption>" + CipherAlg + "</encryption>" +
                            "<useOneX>false</useOneX>" +
                            "</authEncryption>" +
                            "<sharedKey>" +
                            "<keyType>passPhrase</keyType>" +
                            "<protected>false</protected>" +
                            "<keyMaterial>" + edClientPasskey.Text + "</keyMaterial>" +
                        "</sharedKey>" +
                        "</security>" +
                        "</MSM>" +
                        "</WLANProfile>";

                        ShowResult(Client.Connect(Id, wclWiFiConnectionMode.cmTemporaryProfile, Xml,
                            Ssid, ClientGetNetworkBss(), 0, Mac));
                    }
                }
            }
        }

        private wclWiFiBssType ClientGetNetworkBss()
        {
            if (lvClientNetworks.SelectedItems.Count == 0)
                return (wclWiFiBssType)cbClientBssType.SelectedIndex;

            if (((NetworkItem)lvClientNetworks.SelectedItems[0]).BssType == "bssInfrastructure")
                return wclWiFiBssType.bssInfrastructure;

            if (((NetworkItem)lvClientNetworks.SelectedItems[0]).BssType == "bssIndependent")
                return wclWiFiBssType.bssIndependent;

            throw new Exception("Inavlid BSS type");
        }

        private void ClientConnectUsingNetworkProfile()
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                if (lvClientNetworks.SelectedItems.Count == 0)
                    MessageBox.Show("Select network");
                else
                {
                    if (cbUseBssMac.IsChecked == true && lvClientBss.SelectedItems.Count == 0)
                        MessageBox.Show("Select Bss");
                    else
                    {
                        String Mac = "";
                        if (cbUseBssMac.IsChecked == true)
                            Mac = ((BssItem)lvClientBss.SelectedItems[0]).Mac;

                        ShowResult(Client.Connect(Id, wclWiFiConnectionMode.cmProfile,
                            ((NetworkItem)lvClientNetworks.SelectedItems[0]).ProfileName, "", ClientGetNetworkBss(),
                            0, Mac));
                    }
                }
            }
        }

        private void ClientConnectUsingSelectedNetwork()
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                if (lvClientNetworks.SelectedItems.Count == 0)
                    MessageBox.Show("Select network");
                else
                {
                    if (cbUseBssMac.IsChecked == true && lvClientBss.SelectedItems.Count == 0)
                        MessageBox.Show("Select Bss");
                    else
                    {
                        String Mac = "";
                        if (cbUseBssMac.IsChecked == true)
                            Mac = ((BssItem)lvClientBss.SelectedItems[0]).Mac;

                        wclWiFiConnectionMode Mode;
                        if (cbClientDiscoverSecure.IsChecked == true)
                            Mode = wclWiFiConnectionMode.cmDiscoverySecure;
                        else
                            Mode = wclWiFiConnectionMode.cmDiscoveryUnsecure;
                        ShowResult(Client.Connect(Id, Mode, "", ((NetworkItem)lvClientNetworks.SelectedItems[0]).Ssid,
                            ClientGetNetworkBss(), 0, Mac));
                    }
                }
            }
        }

        private void ClientConnectUsingSelectedprofile()
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                if (lvClientNetworks.SelectedItems.Count == 0)
                    MessageBox.Show("Select network");
                else
                {
                    if (lvClientProfiles.SelectedItems.Count == 0)
                        MessageBox.Show("Select profile");
                    else
                    {
                        if (cbUseBssMac.IsChecked == true && lvClientBss.SelectedItems.Count == 0)
                            MessageBox.Show("Select Bss");
                        else
                        {
                            String Mac = "";
                            if (cbUseBssMac.IsChecked == true)
                                Mac = ((BssItem)lvClientBss.SelectedItems[0]).Mac;

                            ShowResult(Client.Connect(Id, wclWiFiConnectionMode.cmProfile,
                                ((NetworkItem)lvClientProfiles.SelectedItems[0]).ProfileName, "", ClientGetNetworkBss(),
                                0, Mac));
                        }
                    }
                }
            }
        }

        private void btTurnOff_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
                ClientSwitchState(Id, true);
        }

        private void btTunrOn_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
                ClientSwitchState(Id, false);
        }

        private void ClientSwitchState(Guid Id, Boolean Off)
        {
            wclWiFiInterface Iface = new wclWiFiInterface(Id);
            try
            {
                if (ShowResult(Iface.Open()))
                {
                    try
                    {
                        if (Off)
                            ShowResult(Iface.TurnOff());
                        else
                            ShowResult(Iface.TurnOn());
                    }
                    finally
                    {
                        Iface.Close();
                    }
                }
            }
            finally
            {
                Iface = null;
            }
        }

        private void btCurrentIp_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    Boolean Static;
                    String Dns1;
                    String Dns2;
                    String Address;
                    String Gateway;
                    String Mask;
                    Int32 Res = Iface.GetCurrentIp(out Static, out Address, out Mask,
                        out Gateway, out Dns1, out Dns2);
                    if (ShowResult(Res))
                    {
                        String Msg;
                        if (Static)
                            Msg = "Static IP\n";
                        else
                            Msg = "DHCP\n";
                        Msg = Msg + "Address: " + Address + "\n" +
                            "Subnet mask: " + Mask + "\n" +
                            "Default gateway: " + Gateway + "\n" +
                            "Name server 1: " + Dns1 + "\n" +
                            "Name server 2: " + Dns2;
                        MessageBox.Show(Msg);
                    }
                    Iface.Close();
                }
                Iface = null;
            }
        }

        private void btGetIpSettings_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    Boolean Static;
                    String Dns1;
                    String Dns2;
                    String Address;
                    String Gateway;
                    String Mask;
                    Int32 Res = Iface.GetIpSettings(out Static, out Address, out Mask,
                        out Gateway, out Dns1, out Dns2);
                    if (ShowResult(Res))
                    {
                        String Msg;
                        if (Static)
                            Msg = "Static IP\n";
                        else
                            Msg = "DHCP\n";
                        Msg = Msg + "Address: " + Address + "\n" +
                            "Subnet mask: " + Mask + "\n" +
                            "Default gateway: " + Gateway + "\n" +
                            "Name server 1: " + Dns1 + "\n" +
                            "Name server 2: " + Dns2;
                        MessageBox.Show(Msg);
                    }
                    Iface.Close();
                }
                Iface = null;
            }
        }

        private void btEnableStaticIp_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    ShowResult(Iface.SetStaticIp("192.168.1.210", "255.255.255.0",
                        "192.168.1.1", "192.168.1.1", ""));
                    Iface.Close();
                }
                Iface = null;
            }
        }

        private void btEnableDhcp_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    ShowResult(Iface.EnableDhcp());
                    Iface.Close();
                }
                Iface = null;
            }
        }

        private void btShowEditUi_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                    ShowResult(ProfilesManager.ShowUIEdit(Id, Profile));
            }
        }

        private void btSetXmlUserData_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                {
                    OpenFileDialog openFileDialog = new OpenFileDialog();
                    openFileDialog.FileName = "";
                    if (openFileDialog.ShowDialog() == true)
                    {
                        String Xml;
                        Xml = System.IO.File.ReadAllText(openFileDialog.FileName);

                        ShowResult(ProfilesManager.SetProfileEapXmlUserData(Id, Profile, false, Xml));
                    }
                }
            }
        }

        private void btGetDualStaState_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    Boolean State;
                    if (ShowResult(Iface.GetSecondarySta(out State)))
                        MessageBox.Show("Dual-STA state: " + State.ToString());
                    Iface.Close();
                }
            }
        }

        private void ClientSetDualState(Boolean Enable)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    ShowResult(Iface.SetSecondarySta(Enable));
                    Iface.Close();
                }
            }
        }

        private void btEnableDualSta_Click(object sender, RoutedEventArgs e)
        {
            ClientSetDualState(true);
        }

        private void btDisableDualSta_Click(object sender, RoutedEventArgs e)
        {
            ClientSetDualState(false);
        }

        private void btEnumSecondaryInterfaces_Click(object sender, RoutedEventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    wclWiFiInterfaceData[] Ifaces;
                    if (ShowResult(Iface.EnumInterfaces(out Ifaces)))
                    {
                        ClientClearInterfaces();
                        if (Ifaces != null)
                        {
                            for (Int32 i = 0; i < Ifaces.Length; i++)
                            {
                                lvClientInterfaces.Items.Add(new InterfaceItem
                                {
                                    InterfaceId = Ifaces[i].Id.ToString(),
                                    Description = Ifaces[i].Description,
                                    Primary = Ifaces[i].Primary.ToString()
                                });
                            }
                        }
                    }
                    Iface.Close();
                }
            }
        }
    }
}
