using System;
using System.Text;
using System.Windows.Forms;

using wclWiFi;
using wclCommon;

namespace WiFiClient
{
    public partial class fmMain : Form
    {
        private wclWiFiClient Client;
        private wclWiFiProfilesManager ProfilesManager;

        public fmMain()
        {
            InitializeComponent();
        }

        private void main_Load(object sender, EventArgs e)
        {
            cbClientBssType.SelectedIndex = 2;
            cbClientConnect.SelectedIndex = 0;

            Client = new wclWiFiClient();
            Client.AfterOpen += new EventHandler(Client_AfterOpen);
            Client.BeforeClose += new EventHandler(Client_BeforeClose);

            ProfilesManager = new wclWiFiProfilesManager();
        }

        void Client_BeforeClose(object sender, EventArgs e)
        {
            ProfilesManager.Close();

            ClientClearInterfaces();
            ClientClearNetworks();
            ClientClearBss();
            ClientClearProfiles();

            MessageBox.Show("WiFi client is closing");
        }

        void Client_AfterOpen(object sender, EventArgs e)
        {
            MessageBox.Show("WiFi client opened");

            ProfilesManager.Open();
        }

        private Boolean ShowResult(Int32 Res)
        {
            if (Res == wclErrors.WCL_E_SUCCESS)
                return true;

            MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            return false;
        }

        private void btClientOpen_Click(object sender, EventArgs e)
        {
            ShowResult(Client.Open());
        }

        private void btClientClose_Click(object sender, EventArgs e)
        {
            ShowResult(Client.Close());
        }

        private void fmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            Client.Close();
        }

        private void ClientClearInterfaces()
        {
            lvClientInterfaces.Items.Clear();
        }

        private void btClientEnumInterfaces_Click(object sender, EventArgs e)
        {
            wclWiFiInterfaceData[] Interfaces = null;
            ClientClearInterfaces();

            if (ShowResult(Client.EnumInterfaces(out Interfaces)))
            {
                if (Interfaces != null)
                {
                    for (Int32 i = 0; i < Interfaces.Length; i++)
                    {
                        ListViewItem Item = lvClientInterfaces.Items.Add(Interfaces[i].Id.ToString());
                        Item.SubItems.Add(Interfaces[i].Description);
                        Item.SubItems.Add(Interfaces[i].Primary.ToString());
                    }
                }
            }
        }

        private Boolean ClientGetInterfaceId(out Guid Id)
        {
            Id = Guid.Empty;

            if (lvClientInterfaces.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select interface");
                return false;
            }

            Id = new Guid(lvClientInterfaces.SelectedItems[0].Text);
            return true;
        }

        private void ClientClearNetworks()
        {
            lvClientNetworks.Items.Clear();
        }

        private void ClientClearBss()
        {
            lvClientBss.Items.Clear();
        }

        private void ClientClearProfiles()
        {
            lvClientProfiles.Items.Clear();
        }

        private void btClientEnumNetworks_Click(object sender, EventArgs e)
        {
            Guid Id = Guid.Empty;

            ClientClearNetworks();
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiAvailableNetworkFilter Filters = 0;
                if (cbClientAdHocProfiles.Checked)
                    Filters |= wclWiFiAvailableNetworkFilter.ffIncludeAllAdhocProfiles;
                if (cbClientHiddenProfiles.Checked)
                    Filters |= wclWiFiAvailableNetworkFilter.ffIncludeAllManualHiddenProfiles;

                wclWiFiAvailableNetwork[] Networks = null;
                if (ShowResult(Client.EnumAvailableNetworks(Id, Filters, out Networks)))
                {
                    if (Networks != null)
                    {
                        for (Int32 i = 0; i < Networks.Length; i++)
                        {
                            ListViewItem Item = lvClientNetworks.Items.Add(Networks[i].ProfileName);
                            Item.SubItems.Add(Networks[i].Ssid);
                            Item.SubItems.Add(Networks[i].BssType.ToString());
                            Item.SubItems.Add(Networks[i].NumberOfBssids.ToString());
                            Item.SubItems.Add(Networks[i].NetworkConnectable.ToString());
                            Item.SubItems.Add(Networks[i].NotConnectableReason.ToString());
                            Item.SubItems.Add(Networks[i].PhyTypes.ToString());
                            Item.SubItems.Add(Networks[i].MorePhyTypes.ToString());
                            Item.SubItems.Add(Networks[i].SignalQuality.ToString());
                            Item.SubItems.Add(Networks[i].SecurityEnabled.ToString());
                            Item.SubItems.Add(Networks[i].DefaultAuthAlgorithm.ToString());
                            Item.SubItems.Add(Networks[i].DefaultCipherAlgorithm.ToString());
                            Item.SubItems.Add(Networks[i].Flags.ToString());
                        }
                    }
                }
            }
        }

        private void btClientScan_Click(object sender, EventArgs e)
        {
            Guid Id = Guid.Empty;
            if (ClientGetInterfaceId(out Id))
            {
                if (ShowResult(Client.Scan(Id, edClientScanSsid.Text)))
                    MessageBox.Show("Scan started. Use EventsDemo to see scan events.");
            }
        }

        private void btClientEnumBss_Click(object sender, EventArgs e)
        {
            ClientClearBss();
            Guid Id = Guid.Empty;

            if (ClientGetInterfaceId(out Id))
            {
                String Ssid = "";

                if (cbClientUseSsid.Checked)
                {
                    if (lvClientNetworks.SelectedItems.Count == 0)
                    {
                        MessageBox.Show("Select network");
                        return;
                    }

                    Ssid = lvClientNetworks.SelectedItems[0].SubItems[1].Text;
                }

                wclWiFiBssType BssType = (wclWiFiBssType)cbClientBssType.SelectedIndex;
                wclWiFiBss[] BssList = null;

                if (ShowResult(Client.EnumBss(Id, Ssid, BssType, cbClientSecurityEnabled.Checked, out BssList)))
                {
                    if (BssList != null)
                    {
                        for (Int32 i = 0; i < BssList.Length; i++)
                        {
                            ListViewItem Item = lvClientBss.Items.Add(BssList[i].Ssid);
                            Item.SubItems.Add(BssList[i].PhyId.ToString());
                            Item.SubItems.Add(BssList[i].Mac);
                            Item.SubItems.Add(BssList[i].BssType.ToString());
                            Item.SubItems.Add(BssList[i].PhyType.ToString());
                            Item.SubItems.Add(BssList[i].Rssi.ToString());
                            Item.SubItems.Add(BssList[i].LinkQuality.ToString());
                            Item.SubItems.Add(BssList[i].InRegDomain.ToString());
                            Item.SubItems.Add(BssList[i].BeaconPeriod.ToString());
                            Item.SubItems.Add(BssList[i].Timestamp.ToString());
                            Item.SubItems.Add(BssList[i].HostTimestamp.ToString());
                            Item.SubItems.Add(BssList[i].Capability.ToString());
                            Item.SubItems.Add(BssList[i].ChCenterFrequency.ToString());

                            String s = "";
                            if (BssList[i].IeRaw != null && BssList[i].IeRaw.Length > 0)
                            {
                                for (Int32 x = 0; x < BssList[i].IeRaw.Length; x++)
                                    s = s + BssList[i].IeRaw[x].ToString("X2");
                            }
                            Item.SubItems.Add(s);
                        }
                    }
                }
            }
        }

        private void btClientDisconnect_Click(object sender, EventArgs e)
        {
            Guid Id = Guid.Empty;
            if (ClientGetInterfaceId(out Id))
                ShowResult(Client.Disconnect(Id));
        }

        private void btClientGetShowDeniedNetworks_Click(object sender, EventArgs e)
        {
            Boolean Enabled = false;
            if (ShowResult(Client.AcmGetShowDeniedNetworks(out Enabled)))
            {
                if (Enabled)
                    MessageBox.Show("Show denied networks");
                else
                    MessageBox.Show("Do not show denied networks");
            }
        }

        private void btClientSetShowDeniedNetworks_Click(object sender, EventArgs e)
        {
            DialogResult Res = MessageBox.Show("Show denied networks?", "Show denied networks",
                MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            if (Res == DialogResult.Cancel)
                return;
            ShowResult(Client.AcmSetShowDeniedNetworks(Res == DialogResult.Yes));
        }

        private void btClientGetPowerSettings_Click(object sender, EventArgs e)
        {
            wclWiFiPowerSetting Setting;
            if (ShowResult(Client.AcmGetPowerSetting(out Setting)))
                MessageBox.Show(Setting.ToString());
        }

        private void btClientGetOnlyUseGpProfiles_Click(object sender, EventArgs e)
        {
            Boolean Enabled;
            if (ShowResult(Client.AcmGetOnlyUseGpProfiles(out Enabled)))
            {
                if (Enabled)
                    MessageBox.Show("Only use GP profiles ENABLED");
                else
                    MessageBox.Show("Only use GP profiles DISABLED");
            }
        }

        private void btClientGetAllowExplicitCreds_Click(object sender, EventArgs e)
        {
            Boolean Enabled;
            if (ShowResult(Client.AcmGetAllowExplicitCreds(out Enabled)))
            {
                if (Enabled)
                    MessageBox.Show("Explicit creds ALLOWED");
                else
                    MessageBox.Show("Explicit creds NOT ALLOWED");
            }
        }

        private void btClientSetAllowExplicitCreds_Click(object sender, EventArgs e)
        {
            DialogResult Res = MessageBox.Show("Allow explicit creds?", "Allow explicit creds?",
                MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            if (Res == DialogResult.Cancel)
                return;
            ShowResult(Client.AcmSetAllowExplicitCreds(Res == DialogResult.Yes));
        }

        private void btClientGetBlockPeriod_Click(object sender, EventArgs e)
        {
            UInt32 Period;
            if (ShowResult(Client.AcmGetBlockPeriod(out Period)))
                MessageBox.Show("Block period: " + Period.ToString());
        }

        private void btClientSetBlockPeriod_Click(object sender, EventArgs e)
        {
            // SET IT TO 10 here
            UInt32 Period = 10;
            ShowResult(Client.AcmSetBlockPeriod(Period));
        }

        private void btClientGetVirtualStationExtensibility_Click(object sender, EventArgs e)
        {
            Boolean Allowed;
            if (ShowResult(Client.AcmGetVirtualStationExtensibility(out Allowed)))
            {
                if (Allowed)
                    MessageBox.Show("Virtual station extensibility ALLOWED");
                else
                    MessageBox.Show("Virtual station extensibility NOT ALLOWED");
            }
        }

        private void btClientSetVirtualStationExtensibility_Click(object sender, EventArgs e)
        {
            DialogResult Res = MessageBox.Show("Allow wirtual station extensibility?",
                "Allow wirtual station extensibility?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            if (Res == DialogResult.Cancel)
                return;

            ShowResult(Client.AcmSetVirtualStationExtensibility(Res == DialogResult.Yes));
        }

        private void btClientGetInterfaceState_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    try
                    {
                        wclWiFiInterfaceState State;
                        if (ShowResult(Iface.GetState(out State)))
                            MessageBox.Show("Interface state: " + State.ToString());
                    }
                    finally
                    {
                        Iface.Close();
                    }
                }
            }
        }

        private void btClientEnumProfiles_Click(object sender, EventArgs e)
        {
            ClientEnumProfiles();
        }

        private void btClientDeleteProfile_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                {
                    if (ShowResult(ProfilesManager.DeleteProfile(Id, Profile)))
                        ClientEnumProfiles();
                }
            }
        }

        private void ClientEnumProfiles()
        {
            ClientClearProfiles();

            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiProfileData[] Profiles;
                if (ShowResult(ProfilesManager.GetProfileList(Id, out Profiles)))
                {
                    if (Profiles != null)
                    {
                        for (Int32 i = 0; i < Profiles.Length; i++)
                        {
                            ListViewItem Item = lvClientProfiles.Items.Add(Profiles[i].Name);
                            Item.SubItems.Add(Profiles[i].Flags.ToString());
                        }
                    }
                }
            }
        }

        private Boolean ClientGetProfile(out String Name)
        {
            Name = "";

            if (lvClientProfiles.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select profile:");
                return false;
            }

            Name = lvClientProfiles.SelectedItems[0].Text;

            return true;
        }

        private void btClientRenameProfile_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                {
                    // Use New_<OLD_PROFILE_NAME>
                    if (ShowResult(ProfilesManager.RenameProfile(Id, Profile, "NEW_" + Profile)))
                        ClientEnumProfiles();
                }
            }
        }

        private void btClientGetprofileXml_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                {
                    wclWiFiProfileFlag Flags = 0;
                    if (cbPlainText.Checked)
                        Flags |= wclWiFiProfileFlag.pfGetPlaintextKey;
                    String Xml;
                    if (ShowResult(ProfilesManager.GetProfile(Id, Profile, ref Flags, out Xml)))
                    {
                        saveFileDialog.FileName = Profile + ".xml";
                        if (saveFileDialog.ShowDialog() == DialogResult.OK)
                            System.IO.File.WriteAllText(saveFileDialog.FileName, Xml);
                    }
                }
            }
        }

        private void btClientSetProfileXml_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                openFileDialog.FileName = "";
                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    String Xml;
                    Xml = System.IO.File.ReadAllText(openFileDialog.FileName);
                    ShowResult(ProfilesManager.SetProfile(Id, 0, Xml, true));
                }
            }
        }

        private void btClientConnect_Click(object sender, EventArgs e)
        {
            switch (cbClientConnect.SelectedIndex)
            {
                case 0:
                    ClientConnectUsingNetworkProfile();
                    break;
                case 1:
                    ClientConnectUsingSelectedprofile();
                    break;
                case 2:
                    ClientConnectUsingSelectedNetwork();
                    break;
                case 3:
                    ClientConenctUsingTemporaryProfile();
                    break;
            }
        }

        private void ClientConenctUsingTemporaryProfile()
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                if (lvClientNetworks.SelectedItems.Count == 0)
                    MessageBox.Show("Select network");
                else
                {
                    if (cbUseBssMac.Checked && lvClientBss.SelectedItems.Count == 0)
                        MessageBox.Show("Select Bss");
                    else
                    {
                        String Mac = "";
                        if (cbUseBssMac.Checked)
                            Mac = lvClientBss.SelectedItems[0].SubItems[2].Text;

                        String Ssid = lvClientNetworks.SelectedItems[0].SubItems[1].Text;

                        Byte[] Utf8 = Encoding.UTF8.GetBytes(Ssid);
                        String Hex = "";
                        foreach (Byte b in Utf8)
                            Hex = Hex + b.ToString("X2");

                        String AuthAlg;
                        if (lvClientNetworks.SelectedItems[0].SubItems[11].Text == "auOpen")
                            AuthAlg = "open";
                        else
                        {
                            if (lvClientNetworks.SelectedItems[0].SubItems[11].Text == "auSharedKey")
                                AuthAlg = "shared";
                            else
                            {
                                if (lvClientNetworks.SelectedItems[0].SubItems[11].Text == "auWpa")
                                    AuthAlg = "WPA";
                                else
                                {
                                    if (lvClientNetworks.SelectedItems[0].SubItems[11].Text == "auWpaPsk")
                                        AuthAlg = "WPAPSK";
                                    else
                                    {
                                        if (lvClientNetworks.SelectedItems[0].SubItems[11].Text == "auRsna")
                                            AuthAlg = "WPA2";
                                            else
                                        {
                                            if (lvClientNetworks.SelectedItems[0].SubItems[11].Text == "auWpa3")
                                                AuthAlg = "WPA3ENT192";
                                            else
                                            {
                                                if (lvClientNetworks.SelectedItems[0].SubItems[11].Text == "auWpa3Sae")
                                                    AuthAlg = "WPA3SAE";
                                                else
                                                {
                                                    if (lvClientNetworks.SelectedItems[0].SubItems[11].Text == "auWpa3Ent")
                                                        AuthAlg = "WPA3ENT";
                                                    else
                                                    {
                                                        if (lvClientNetworks.SelectedItems[0].SubItems[11].Text == "auOwe")
                                                            AuthAlg = "OWE";
                                                        else
                                                            AuthAlg = "WPA2PSK";
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        String CipherAlg;
                        if (lvClientNetworks.SelectedItems[0].SubItems[12].Text == "caNone")
                            CipherAlg = "none";
                        else
                        {
                            if (lvClientNetworks.SelectedItems[0].SubItems[12].Text == "caTkip")
                                CipherAlg = "TKIP";
                            else
                            {
                                if (lvClientNetworks.SelectedItems[0].SubItems[12].Text == "caWep40" ||
                                    lvClientNetworks.SelectedItems[0].SubItems[12].Text == "caWep104" ||
                                    lvClientNetworks.SelectedItems[0].SubItems[12].Text == "caWep")
                                {
                                    CipherAlg = "WEP";
                                }
                                else
                                {
                                    if (lvClientNetworks.SelectedItems[0].SubItems[12].Text == "caGcmp256")
                                        CipherAlg = "GCMP256";
                                    else
                                        CipherAlg = "AES";
                                }
                            }
                        }

                        String Xml = "<?xml version=\"1.0\"?>" +
                            "<WLANProfile xmlns=\"http://www.microsoft.com/networking/WLAN/profile/v1\">" +
                            "<name>" + Ssid + "</name>" +
                            "<SSIDConfig>" +
                            "<SSID>" +
                            "<hex>" + Hex + "</hex>" +
                            "<name>" + Ssid + "</name>" +
                            "</SSID>" +
                            "</SSIDConfig>" +
                            "<connectionType>ESS</connectionType>" +
                            "<connectionMode>manual</connectionMode>" +
                            "<autoSwitch>false</autoSwitch>" +
                            "<MSM>" +
                            "<security>" +
                            "<authEncryption>" +
                            "<authentication>" + AuthAlg + "</authentication>" +
                            "<encryption>" + CipherAlg + "</encryption>" +
                            "<useOneX>false</useOneX>" +
                            "</authEncryption>" +
                            "<sharedKey>" +
                            "<keyType>passPhrase</keyType>" +
                            "<protected>false</protected>" +
                            "<keyMaterial>" + edClientPasskey.Text + "</keyMaterial>" +
                            "</sharedKey>" +
                            "</security>" +
                            "</MSM>" +
                            "</WLANProfile>";

                        ShowResult(Client.Connect(Id, wclWiFiConnectionMode.cmTemporaryProfile, Xml,
                            Ssid, ClientGetNetworkBss(), 0, Mac));
                    }
                }
            }
        }

        private wclWiFiBssType ClientGetNetworkBss()
        {
            if (lvClientNetworks.SelectedItems.Count == 0)
                return (wclWiFiBssType)cbClientBssType.SelectedIndex;

            if (lvClientNetworks.SelectedItems[0].SubItems[2].Text == "bssInfrastructure")
                return wclWiFiBssType.bssInfrastructure;

            if (lvClientNetworks.SelectedItems[0].SubItems[2].Text == "bssIndependent")
                return wclWiFiBssType.bssIndependent;

            throw new Exception("Inavlid BSS type");
        }

        private void ClientConnectUsingNetworkProfile()
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                if (lvClientNetworks.SelectedItems.Count == 0)
                    MessageBox.Show("Select network");
                else
                {
                    if (cbUseBssMac.Checked && lvClientBss.SelectedItems.Count == 0)
                        MessageBox.Show("Select Bss");
                    else
                    {
                        String Mac = "";
                        if (cbUseBssMac.Checked)
                            Mac = lvClientBss.SelectedItems[0].SubItems[2].Text;

                        ShowResult(Client.Connect(Id, wclWiFiConnectionMode.cmProfile,
                            lvClientNetworks.SelectedItems[0].Text, "", ClientGetNetworkBss(),
                            0, Mac));
                    }
                }
            }
        }

        private void ClientConnectUsingSelectedNetwork()
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                if (lvClientNetworks.SelectedItems.Count == 0)
                    MessageBox.Show("Select network");
                else
                {
                    if (cbUseBssMac.Checked && lvClientBss.SelectedItems.Count == 0)
                        MessageBox.Show("Select Bss");
                    else
                    {
                        String Mac = "";
                        if (cbUseBssMac.Checked)
                            Mac = lvClientBss.SelectedItems[0].SubItems[2].Text;

                        wclWiFiConnectionMode Mode;
                        if (cbClientDiscoverSecure.Checked)
                            Mode = wclWiFiConnectionMode.cmDiscoverySecure;
                        else
                            Mode = wclWiFiConnectionMode.cmDiscoveryUnsecure;
                        ShowResult(Client.Connect(Id, Mode, "", lvClientNetworks.SelectedItems[0].SubItems[1].Text,
                            ClientGetNetworkBss(), 0, Mac));
                    }
                }
            }
        }

        private void ClientConnectUsingSelectedprofile()
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                if (lvClientNetworks.SelectedItems.Count == 0)
                    MessageBox.Show("Select network");
                else
                {
                    if (lvClientProfiles.SelectedItems.Count == 0)
                        MessageBox.Show("Select profile");
                    else
                    {
                        if (cbUseBssMac.Checked && lvClientBss.SelectedItems.Count == 0)
                            MessageBox.Show("Select Bss");
                        else
                        {
                            String Mac = "";
                            if (cbUseBssMac.Checked)
                                Mac = lvClientBss.SelectedItems[0].SubItems[2].Text;

                            ShowResult(Client.Connect(Id, wclWiFiConnectionMode.cmProfile,
                                lvClientProfiles.SelectedItems[0].Text, "", ClientGetNetworkBss(),
                                0, Mac));
                        }
                    }
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
                ClientSwitchState(Id, true);
        }

        private void btTurnOn_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
                ClientSwitchState(Id, false);
        }

        private void ClientSwitchState(Guid Id, Boolean Off)
        {
            wclWiFiInterface Iface = new wclWiFiInterface(Id);
            try
            {
                if (ShowResult(Iface.Open()))
                {
                    try
                    {
                        if (Off)
                            ShowResult(Iface.TurnOff());
                        else
                            ShowResult(Iface.TurnOn());
                    }
                    finally
                    {
                        Iface.Close();
                    }
                }
            }
            finally
            {
                Iface = null;
            }
        }

        private void btCurrentIp_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    Boolean Static;
                    String Dns1;
                    String Dns2;
                    String Address;
                    String Gateway;
                    String Mask;
                    Int32 Res = Iface.GetCurrentIp(out Static, out Address, out Mask,
                        out Gateway, out Dns1, out Dns2);
                    if (ShowResult(Res))
                    {
                        String Msg;
                        if (Static)
                            Msg = "Static IP\n";
                        else
                            Msg = "DHCP\n";
                        Msg = Msg + "Address: " + Address + "\n" +
                            "Subnet mask: " + Mask + "\n" +
                            "Default gateway: " + Gateway + "\n" +
                            "Name server 1: " + Dns1 + "\n" +
                            "Name server 2: " + Dns2;
                        MessageBox.Show(Msg);
                    }
                    Iface.Close();
                }
                Iface = null;
            }
        }

        private void btGetIpSettings_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    Boolean Static;
                    String Dns1;
                    String Dns2;
                    String Address;
                    String Gateway;
                    String Mask;
                    Int32 Res = Iface.GetIpSettings(out Static, out Address, out Mask,
                        out Gateway, out Dns1, out Dns2);
                    if (ShowResult(Res))
                    {
                        String Msg;
                        if (Static)
                            Msg = "Static IP\n";
                        else
                            Msg = "DHCP\n";
                        Msg = Msg + "Address: " + Address + "\n" +
                            "Subnet mask: " + Mask + "\n" +
                            "Default gateway: " + Gateway + "\n" +
                            "Name server 1: " + Dns1 + "\n" +
                            "Name server 2: " + Dns2;
                        MessageBox.Show(Msg);
                    }
                    Iface.Close();
                }
                Iface = null;
            }
        }

        private void btEnableDhcp_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    ShowResult(Iface.EnableDhcp());
                    Iface.Close();
                }
                Iface = null;
            }
        }

        private void btEnableStaticIp_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    ShowResult(Iface.SetStaticIp("192.168.1.210", "255.255.255.0",
                        "192.168.1.1", "192.168.1.1", ""));
                    Iface.Close();
                }
                Iface = null;
            }
        }

        private void btShowEditUi_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                    ShowResult(ProfilesManager.ShowUIEdit(Id, Profile));
            }
        }

        private void btSetXmlUserData_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                String Profile;
                if (ClientGetProfile(out Profile))
                {
                    openFileDialog.FileName = "";
                    if (openFileDialog.ShowDialog() == DialogResult.OK)
                    {
                        String Xml;
                        Xml = System.IO.File.ReadAllText(openFileDialog.FileName);

                        ShowResult(ProfilesManager.SetProfileEapXmlUserData(Id, Profile, false, Xml));
                    }
                }
            }
        }

        private void btGetDualStaState_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    Boolean State;
                    if (ShowResult(Iface.GetSecondarySta(out State)))
                        MessageBox.Show("Dual-STA state: " + State.ToString());
                    Iface.Close();
                }
            }
        }

        private void ClientSetDualState(Boolean Enable)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    ShowResult(Iface.SetSecondarySta(Enable));
                    Iface.Close();
                }
            }
        }

        private void btEnableDualSta_Click(object sender, EventArgs e)
        {
            ClientSetDualState(true);
        }

        private void btDisableDualSta_Click(object sender, EventArgs e)
        {
            ClientSetDualState(false);
        }

        private void btEnumSecondaryInterfaces_Click(object sender, EventArgs e)
        {
            Guid Id;
            if (ClientGetInterfaceId(out Id))
            {
                wclWiFiInterface Iface = new wclWiFiInterface(Id);
                if (ShowResult(Iface.Open()))
                {
                    wclWiFiInterfaceData[] Ifaces;
                    if (ShowResult(Iface.EnumInterfaces(out Ifaces)))
                    {
                        ClientClearInterfaces();
                        if (Ifaces != null)
                        {
                            for (Int32 i = 0; i < Ifaces.Length; i++)
                            {
                                ListViewItem Item = lvClientInterfaces.Items.Add(Ifaces[i].Id.ToString());
                                Item.SubItems.Add(Ifaces[i].Description);
                                Item.SubItems.Add(Ifaces[i].Primary.ToString());
                            }
                        }
                    }
                    Iface.Close();
                }
            }
        }
    }
}